#-*- coding: utf-8 -*-
"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""

import random
import string
from cocktail import schema
from cocktail.translations import translations
from woost.models import Extension

translations.define("SignUpExtension",
    ca = u"Alta d'usuaris",
    es = u"Alta de usuarios",
    en = u"Sign Up"
)

translations.define("SignUpExtension-plural",
    ca = u"Altas d'usuaris",
    es = u"Altas de usuarios",
    en = u"Signs Up"
)

class SignUpExtension(Extension):

    # To indicate if the extension was loaded at least one time
    # will be set to True at the end of the first load
    initialized = False

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet als usuaris registrar-se autònomament en el lloc web""",
            "ca"
        )
        self.set("description",
            u"""Permite a los usuarios registrarse por si mismos en el sitio""",
            "es"
        )
        self.set("description",
            u"""Allows users to register themselves on the site""",
            "en"
        )
        self.secret_key = self._generate_secret_key()

    def _generate_secret_key(self):
        secret_key = ''
        for i in range(0,8):
            secret_key = secret_key + \
                random.choice(
                    string.letters + string.digits
                )
        return secret_key

    def _load(self):

        from woost.models import User
        from woost.extensions.signup import (
            signuppage,
            signupcontroller,
            strings
        )

        # Extend User model
        if not hasattr(User, "confirmed_email"):
            User.add_member(
                schema.Boolean(
                    "confirmed_email",
                    default = False,
                    Required = True
                )
            )

        self.install()

    def _install(self):

        from woost.models import (
            extension_translations,
            User,
            Controller,
            Document,
            StandardPage,
            Template,
            EmailTemplate
        )
        from woost.extensions.signup.signuppage import SignUpPage

        signup_controller = self._create_asset(
            Controller,
            "signup_controller",
            python_name =
                "woost.extensions.signup.signupcontroller.SignUpController",
            title = extension_translations
        )

        signup_confirmation_controller = self._create_asset(
            Controller,
            "signup_confirmation_controller",
            python_name =
                "woost.extensions.signup.signupconfirmationcontroller."
                "SignUpConfirmationController",
            title = extension_translations
        )

        signup_view = self._create_asset(
            Template,
            "signup_template",
            identifier = "woost.extensions.signup.SignUpView",
            engine = u"cocktail",
            title = extension_translations
        )

        signup_confirmation_view = self._create_asset(
            Template,
            "signup_confirmation_template",
            identifier = u"woost.extensions.signup.SignUpConfirmationView",
            engine = u"cocktail",
            title = extension_translations
        )

        confirmation_email_template = self._create_asset(
            EmailTemplate,
            "signup_confirmation_email_template",
            python_name =
                u"woost.extensions.signup.signupconfirmationemailtemplate."
                "SignUpConfirmationEmailTemplate",
            template_engine = u"mako",
            sender = User.require_instance(qname="woost.administrator").email,
            receivers = u"[user.email]",
            title = extension_translations,
            subject = extension_translations,
            body = extension_translations
        )

        confirmation_target = self._create_asset(
            StandardPage,
            "signup_confirmation_target",
            title = extension_translations,
            controller = signup_confirmation_controller,
            template = signup_confirmation_view,
            hidden = True
        )

        signup_page = self._create_asset(
            SignUpPage,
            "signup_page",
            title = extension_translations,
            user_type = User,
            confirmation_target = confirmation_target,
            parent = Document.get_instance(qname="woost.home"),
        )

        signup_page.children.append(confirmation_target)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			March 2009
"""
from cocktail.html import templates
from woost.models import get_current_user, ReadPermission

TreeView = templates.get_class("cocktail.html.TreeView")


class ContentTreeView(TreeView):

    base_url = None
    edit_stack = None

    def filter(self, item):
        return get_current_user().has_permission(ReadPermission, target = item)

    def create_label(self, item):
        label = templates.new("woost.views.ContentLink")
        label.base_url = self.base_url
        label.item = item
        label.edit_stack = self.edit_stack
        return label




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from __future__ import with_statement
import cherrypy
from cocktail import schema
from cocktail.events import when
from cocktail.translations import translations
from cocktail.controllers import Location
from woost.models import Extension

translations.define("CommentsExtension",
    ca = u"Comentaris",
    es = u"Comentarios",
    en = u"Comments"
)

translations.define("CommentsExtension-plural",
    ca = u"Comentaris",
    es = u"Comentarios",
    en = u"Comments"
)

translations.define("CommentsExtension.captcha_enabled",
    ca = u"Captcha activat",
    es = u"Captcha activado",
    en = u"Captcha enabled"
)


class CommentsExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Afegeix suport per comentaris.""",
            "ca"
        )
        self.set("description",
            u"""Añade soporte para comentarios.""",
            "es"
        )
        self.set("description",
            u"""Adds support for comments.""",
            "en"
        )

    def _create_comments_adapter(self, comment_model):
        adapter = schema.Adapter()
        adapter.exclude(
            member.name
            for member in comment_model.members().itervalues()
            if not member.visible
            or not member.editable
            or not issubclass(member.schema, comment_model)
        )
        adapter.exclude(["publishable","captcha"])
        return adapter

    def _adapt_comments_schema(self, comment_model):
        adapter = self._create_comments_adapter(comment_model)
        comments_schema = schema.Schema(comment_model.name + "Form")
        adapter.export_schema(comment_model, comments_schema)
        return comments_schema

    def _after_process_comments(self, comment):
        raise cherrypy.HTTPRedirect(
            "%s#comment-%s" % (
                unicode(Location.get_current()).encode('utf-8'),
                str(comment.id)
            )
        )

    def _load(self):

        from cocktail.persistence import datastore
        from cocktail.controllers import UserCollection, get_parameter
        from cocktail.pkgutils import resolve
        from woost.models import (
            Publishable,
            Role,
            CreatePermission,
            get_current_user
        )
        from woost.models.changesets import changeset_context
        from woost.controllers.basecmscontroller import BaseCMSController

        # Import the extension's models
        from woost.extensions.comments import strings
        from woost.extensions.comments.comment import Comment

        # Captcha
        #------------------------------------------------------------------------------
        try:
            from woost.extensions.recaptcha import ReCaptchaExtension
        except ImportError:
            pass
        else:
            from woost.extensions.recaptcha.schemarecaptchas import ReCaptcha
            if ReCaptchaExtension.instance.enabled:
                CommentsExtension.add_member(
                    schema.Boolean("captcha_enabled", default = False)
                )

        # Extend Publishable model
        Publishable.add_member(
            schema.Boolean(
                "allow_comments",
                required = True,
                default = False,
                member_group = "administration"
            )
        )

        Publishable.add_member(
            schema.Collection(
                "comments",
                items = schema.Reference(type = Comment),
                bidirectional = True,
                related_key = "publishable",
                integral = True
            )
        )

        @when(BaseCMSController.processed)
        def _process_comments(event):

            # Comments variables initialization
            comments_user_collection = None
            comments_schema = None
            comment_errors = None
            comment_data = {}

            controller = event.source
            comment_model = \
                resolve(getattr(controller, "comment_model", Comment))
            publishable = controller.context["publishable"]
            user = get_current_user()

            if publishable is not None and publishable.allow_comments:

                # Comments collection
                comments_user_collection = UserCollection(comment_model)
                comments_user_collection.allow_type_selection = False
                comments_user_collection.allow_filters = False
                comments_user_collection.allow_language_selection = False
                comments_user_collection.allow_member_selection = False
                comments_user_collection.params.prefix = "comments_"
                comments_user_collection.base_collection = publishable.comments

                # Show the last comments page if not specified
                if "comments_page" not in cherrypy.request.params:
                    div, mod = divmod(
                        len(comments_user_collection.subset),
                        comments_user_collection.page_size
                    )
                    comments_page = div - 1 if not mod and div != 0 else div
                    cherrypy.request.params.update(
                        comments_page = str(comments_page)
                    )

                # Adapting the comments model
                comments_schema = CommentsExtension.instance._adapt_comments_schema(comment_model)

                if user.anonymous \
                and getattr(CommentsExtension.instance, "captcha_enabled", False):
                    comments_schema.add_member(
                        ReCaptcha("captcha")
                    )

                # Insert a new comment
                if cherrypy.request.method == "POST" \
                and "post_comment" in cherrypy.request.params:

                    with changeset_context(user):
                        get_parameter(
                            comments_schema,
                            target = comment_data,
                            errors = "ignore"
                        )

                        comment_errors = schema.ErrorList(
                            comments_schema.get_errors(comment_data)
                        )

                        if not comment_errors:
                            comment = comment_model()

                            adapter = CommentsExtension.instance._create_comments_adapter(comment_model)
                            adapter.import_object(
                                comment_data,
                                comment,
                                source_schema = comments_schema,
                                target_schema = comment_model
                            )

                            comment.publishable = publishable
                            user.require_permission(CreatePermission, target = comment)

                            comment.insert()
                            datastore.commit()
                            CommentsExtension.instance._after_process_comments(comment)
                else:
                    comment_errors = schema.ErrorList([])

            # Update the output
            controller.output.update(
                comments_user_collection = comments_user_collection,
                comments_schema = comments_schema,
                comment_errors = comment_errors,
                comment_data = comment_data
            )

        self.install()

    def _install(self):
        self._create_asset(
            CreatePermission,
            "anonymous_comments_permission",
            role = Role.require_instance(qname = "woost.anonymous"),
            authorized = True,
            matching_items = {
                "type": "woost.extensions.comments.comment.Comment"
            }
        )




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from cocktail.tests.seleniumtester import selenium_test, browser
from woost.models import Item
from woost.tests.test_selenium import admin_login


class FlatContentViewTestCase(object):

    @selenium_test
    def test_show_all(self):
        all = len(Item.select())
        browser.open("/en/cms/content/?content_view=flat&page_size=%d" % all)
        admin_login()
        assert browser.jquery_count(".collection_display tr.item_row") == all

    @selenium_test
    def test_sort_by_id(self):

        browser.open(
            "/en/cms/content/?content_view=flat"
            "&page_size=%d"
            "&members=element&members=id"
            % len(Item.select())
        )
        admin_login()

        def get_browser_ids():
            return browser.get_eval(
                "var ids = [];"
                "window.jQuery('.collection_display tbody td.id_column')"
                ".each(function () {"
                "   ids.push(window.jQuery(this).text());"
                "});"
                "ids.join(' ');"
            )

        # Show the column dropdown panel and click the ascending order button
        browser.click("css=.collection_display th.id_column .selector a.label")
        browser.click(
            "css=.collection_display th.id_column .sorting_options "
            "a.ascending"
        )
        browser.wait_for_page_to_load(10000)
        sorted_ids = " ".join(str(x.id) for x in Item.select(order = "id"))
        assert get_browser_ids() == sorted_ids

        # Show the column dropdown panel and click the descending order button
        browser.click("css=.collection_display th.id_column .selector a.label")
        browser.click(
            "css=.collection_display th.id_column .sorting_options "
            "a.descending"
        )
        browser.wait_for_page_to_load(10000)
        sorted_ids = " ".join(str(x.id) for x in Item.select(order = "-id"))
        assert get_browser_ids() == sorted_ids

    @selenium_test
    def test_selection_tools(self):

        browser.open("/en/cms/content/?content_view")
        admin_login()

        assert browser.is_element_present("css=.select_all")
        assert browser.is_element_present("css=.clear_selection")

        browser.click("css=.select_all")
        all = browser.jquery_count(".collection_display tr.item_row")
        sel = browser.jquery_count(".collection_display tr.item_row.selected")
        assert all == sel

        browser.click("css=.clear_selection")
        sel = browser.jquery_count(".collection_display tr.item_row.selected")
        assert sel == 0

    @selenium_test
    def test_member_selection(self):

        browser.open("/en/cms/content/?content_view")
        admin_login()

        # Unfold the collection settings panel
        browser.click("css=.settings_box a.label")

        all_columns = ["element", "class"] + [
            member.name
            for member in Item.members().itervalues()
            if member.visible
        ]

        for visible_members in (
            set(["element"]),
            set(["element", "class"]),
            set(["element", "class", "id"]),
            set(["id", "author", "owner", "is_draft", "drafts"]),
            set(["element", "translations"])
        ):
            # Check / uncheck columns
            for key in all_columns:
                locator = \
                    "css=.settings_box .members_selector input[value=%s]" % key
                if key in visible_members:
                    browser.check(locator)
                else:
                    browser.uncheck(locator)

            # Apply the member selection
            browser.click("css=.settings_box button[type=submit]")
            browser.wait_for_page_to_load(10000)

            # Assert the browser shows the required columns
            for key in all_columns:
                check_locator = \
                    "css=.settings_box .members_selector input[value=%s]" % key
                column_locator = \
                    "css=.collection_display th.%s_column" % key
                if key in visible_members:
                    assert browser.is_checked(check_locator)
                    assert browser.is_visible(column_locator)
                else:
                    assert not browser.is_checked(check_locator)
                    assert not browser.is_element_present(column_locator)

    @selenium_test
    def test_simple_search(self):

        browser.open("/en/cms/content/?content_view")
        admin_login()

        # Fill in and submit the simple search box
        query = "administrators"
        browser.type("filter_value0", query)
        browser.click("css=.search_button")
        browser.wait_for_page_to_load(10000)

        # Make sure this turns on the advanced search panel
        assert not browser.is_element_present("css=.advanced_search")
        assert browser.is_element_present("css=.filters")
        assert browser.is_element_present("filter_value0")
        assert browser.get_value("filter_value0") == query

        # Test returned results
        assert browser.jquery_count(".collection_display tr.item_row") == 1

    @selenium_test
    def test_simple_search_without_query(self):

        browser.open("/en/cms/content/?content_view")
        admin_login()

        # Perform a search without query
        browser.click("css=.search_button")
        browser.wait_for_page_to_load(10000)

        # Make sure this doesn't turn on the advanced search panel
        assert browser.is_element_present("css=.advanced_search")




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from __future__ import with_statement
from woost.tests.models.basetestcase import BaseTestCase


class ChangeSetTests(BaseTestCase):

    def test_insert(self):

        from datetime import datetime
        from woost.models import (
            Document, User, ChangeSet, changeset_context
        )

        author = User()
        author.insert()

        with changeset_context(author) as changeset:
            item = Document()
            item.set("title", u"Foo!", "en")
            item.resource_type = u"text/foo"
            item.hidden = True
            assert not changeset.changes
            item.insert()

        assert list(ChangeSet.select()) == [changeset]
        assert changeset.author is author
        assert isinstance(changeset.date, datetime)

        assert changeset.changes.keys() == [item.id]
        change = changeset.changes[item.id]
        assert change.target is item
        assert change.action is self.create_action
        assert change.changeset is changeset
        assert item.changes == [change]

        for key in "id", "changes", "creation_time", "last_update_time":
            assert not key in change.item_state

        assert change.item_state["title"] == {"en": u"Foo!"}
        assert change.item_state["resource_type"] == u"text/foo"
        assert change.item_state["hidden"] == True
        assert change.item_state["translation_enabled"] == \
            {"en": item.get("translation_enabled", "en")}

        assert item.author is author
        assert item.owner is author
        assert item.creation_time
        assert item.last_update_time
        assert item.creation_time == item.last_update_time

    def test_delete(self):

        from datetime import datetime
        from woost.models import (
            Item, User, ChangeSet, changeset_context
        )

        author = User()
        author.insert()

        item = Item()
        item.insert()

        with changeset_context(author) as changeset:
            item.owner = None
            item.delete()

        assert list(ChangeSet.select()) == [changeset]
        assert changeset.author is author
        assert isinstance(changeset.date, datetime)

        assert changeset.changes.keys() == [item.id]
        change = changeset.changes[item.id]
        assert change.target is item
        assert change.action is self.delete_action
        assert change.changeset is changeset
        assert item.changes == [change]

        assert not item.id in Item.index

    def test_modify(self):

        from time import sleep
        from datetime import datetime
        from woost.models import (
            Document, User, ChangeSet, changeset_context
        )

        author = User()
        author.insert()

        with changeset_context(author) as creation:
            item = Document()
            item.set("title", u"Foo!", "en")
            item.resource_type = u"text/foo"
            item.hidden = True
            item.insert()

        # Make sure creation_time and last_update_time don't match
        sleep(0.1)

        with changeset_context(author) as modification:
            item.set("title", u"Bar!", "en")
            item.resource_type = u"text/bar"
            item.hidden = True

        assert list(ChangeSet.select()) == [creation, modification]
        assert modification.author is author
        assert isinstance(modification.date, datetime)

        assert modification.changes.keys() == [item.id]
        change = modification.changes[item.id]
        assert change.target is item
        assert change.action is self.modify_action
        assert change.changeset is modification
        assert change.changed_members == set(["title", "resource_type"])
        assert item.changes == [creation.changes[item.id], change]

        for key in "id", "changes", "creation_time", "last_update_time":
            assert not key in change.item_state

        assert change.item_state["title"] == {"en": u"Bar!"}
        assert change.item_state["resource_type"] == u"text/bar"
        assert change.item_state["hidden"] == True
        assert change.item_state["translation_enabled"] == \
            {"en": item.get("translation_enabled", "en")}

        assert item.author is author
        assert item.owner is author
        assert item.creation_time
        assert item.last_update_time
        assert not item.creation_time == item.last_update_time




#!/usr/bin/env python
#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from woost.controllers.installer import Installer

def main():
    installer = Installer()
    installer.run()

if __name__ == "__main__":
    main()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from cocktail.translations import translations

# Payments extension
#------------------------------------------------------------------------------
translations.define("PaymentsExtension.payment_gateway",
    ca = u"Passarel·la de pagaments",
    es = u"Pasarela de pagos",
    en = u"Payment gateway"
)

# TransactionNotifiedTrigger
#------------------------------------------------------------------------------
translations.define("TransactionNotifiedTrigger",
    ca = u"Disparador de notificació de pagament",
    es = u"Disparador de notificación de pago",
    en = u"Payment notification trigger"
)

translations.define("TransactionNotifiedTrigger-plural",
    ca = u"Disparadors de notificació de pagament",
    es = u"Disparadores de notificación de pago",
    en = u"Payment notification triggers"
)

# PaymentGateway
#------------------------------------------------------------------------------
translations.define("PaymentGateway",
    ca = u"Passarel·la de pagaments",
    es = u"Pasarela de pagos",
    en = u"Payment gateway"
)

translations.define("PaymentGateway-plural",
    ca = u"Passarel·les de pagaments",
    es = u"Pasarelas de pagos",
    en = u"Payment gateways"
)

translations.define("PaymentGateway.label",
    ca = u"Etiqueta",
    es = u"Etiqueta",
    en = u"Label"
)

translations.define("PaymentGateway.label-explanation",
    ca = u"Etiqueta utilitzada per mostrar als usuaris",
    es = u"Etiqueta utilizada para mostrar a los usuarios",
    en = u"Label showed to the users"
)

translations.define("PaymentGateway.test_mode",
    ca = u"Mode de proves",
    es = u"Modo de pruebas",
    en = u"Test mode"
)

translations.define("PaymentGateway.currency",
    ca = u"Divisa",
    es = u"Divisa",
    en = u"Currency"
)

# Pasat4bPaymentGateway
#------------------------------------------------------------------------------
translations.define("Pasat4bPaymentGateway",
    ca = u"Pasat 4B",
    es = u"Pasat 4B",
    en = u"Pasat 4B"
)

translations.define("Pasat4bPaymentGateway-plural",
    ca = u"Pasat 4B",
    es = u"Pasat 4B",
    en = u"Pasat 4B"
)

translations.define("Pasat4bPaymentGateway.merchant_code",
    ca = u"Codi de comerç",
    es = u"Código de comercio",
    en = u"Merchant code"
)

translations.define("Pasat4bPaymentGateway.label default",
    ca = u"Targeta de crèdit",
    es = u"Tarjeta de crédito",
    en = u"Credit card"
)

# SISPaymentGateway
#------------------------------------------------------------------------------
translations.define("SISPaymentGateway",
    ca = u"SIS/Sermepa",
    es = u"SIS/Sermepa",
    en = u"SIS/Sermepa"
)

translations.define("SISPaymentGateway-plural",
    ca = u"SIS/Sermepa",
    es = u"SIS/Sermepa",
    en = u"SIS/Sermepa"
)

translations.define("SISPaymentGateway.merchant_code",
    ca = u"Codi de comerç",
    es = u"Código de comercio",
    en = u"Merchant code"
)

translations.define("SISPaymentGateway.merchant_name",
    ca = u"Nom del comerç",
    es = u"Nombre del comercio",
    en = u"Merchant name"
)

translations.define("SISPaymentGateway.merchant_terminal",
    ca = u"Número de terminal",
    es = u"Número de terminal",
    en = u"Terminal number"
)

translations.define("SISPaymentGateway.merchant_secret_key",
    ca = u"Clau secreta",
    es = u"Clave secreta",
    en = u"Secret key"
)

translations.define("SISPaymentGateway.payment_successful_page",
    ca = u"Pàgina de confirmació de pagament",
    es = u"Página de confirmación de pago",
    en = u"Payment successful page"
)

translations.define("SISPaymentGateway.payment_failed_page",
    ca = u"Pàgina de pagament fallit",
    es = u"Página de pago fallido",
    en = u"Payment failed page"
)

translations.define("SISPaymentGateway.label default",
    ca = u"Targeta de crèdit",
    es = u"Tarjeta de crédito",
    en = u"Credit card"
)

# DummyPaymentGateway
#------------------------------------------------------------------------------
translations.define("DummyPaymentGateway",
    ca = u"Passarel·la de pagaments fictícia",
    es = u"Pasarela de pagos ficticia",
    en = u"Dummy payment gateway"
)

translations.define("DummyPaymentGateway-plural",
    ca = u"Passarel·les de pagaments fictícies",
    es = u"Pasarelas de pagos ficticias",
    en = u"Dummy payment gateways"
)

translations.define("DummyPaymentGateway.payment_status",
    ca = u"Resultat dels pagaments",
    es = u"Resultado de los pagos",
    en = u"Payment outcome"
)

translations.define("DummyPaymentGateway.payment_successful_page",
    ca = u"Pàgina de confirmació de pagament",
    es = u"Página de confirmación de pago",
    en = u"Payment successful page"
)

translations.define("DummyPaymentGateway.payment_failed_page",
    ca = u"Pàgina de pagament fallit",
    es = u"Página de pago fallido",
    en = u"Payment failed page"
)

translations.define(
    "woost.extensions.payments.DummyPaymentGateway.payment_status "
    "accepted",
    ca = u"Acceptat",
    es = u"Aceptado",
    en = u"Accepted"
)

translations.define(
    "woost.extensions.payments.DummyPaymentGateway.payment_status "
    "failed",
    ca = u"Cancel·lat",
    es = u"Cancelado",
    en = u"Canceled"
)

# PayPalPaymentGateway
#------------------------------------------------------------------------------
translations.define("PayPalPaymentGateway",
    ca = u"PayPal",
    es = u"PayPal",
    en = u"PayPal"
)

translations.define("PayPalPaymentGateway-plural",
    ca = u"PayPal",
    es = u"PayPal",
    en = u"PayPal"
)

translations.define("PayPalPaymentGateway.business",
    ca = u"Compte de PayPal",
    es = u"Cuenta de PayPal",
    en = u"PayPal account"
)

translations.define("PayPalPaymentGateway.payment_successful_page",
    ca = u"Pàgina de confirmació de pagament",
    es = u"Página de confirmación de pago",
    en = u"Payment successful page"
)

translations.define("PayPalPaymentGateway.payment_failed_page",
    ca = u"Pàgina de pagament fallit",
    es = u"Página de pago fallido",
    en = u"Payment failed page"
)

translations.define("PayPalPaymentGateway.label default",
    ca = u"Targeta de crèdit / PayPal",
    es = u"Tarjeta de crédito / PayPal",
    en = u"Credit card / PayPal"
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
import __builtin__
from datetime import datetime
import cherrypy
from cocktail.controllers.location import Location
from cocktail.translations import translations, get_language
from woost.models import (
    Feed,
    Site,
    PermissionExpression,
    ReadPermission,
    get_current_user
)
from woost.controllers.publishablecontroller import PublishableController


class FeedController(PublishableController):

    def _produce_content(self, **kwargs):

        site = Site.main
        cms = self.context["cms"]
        feed = self.context["publishable"]

        location = Location.get_current()
        location.relative = False

        def rfc822_date(date):
            tz = "Z" if site.timezone is None else site.timezone
            return date.strftime("%d %%s %Y %H:%M:%S %%s") % (
                ("Jan", "Feb", "Mar", "Apr", "May", "Jun",
                 "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")[date.month - 1],
                tz
            )

        params = {
            "title": feed.title,
            "url": unicode(location),
            "description": feed.description,
            "language": get_language(),
            "now": rfc822_date(datetime.now())
        }

        location.path_info = self.application_uri()
        location.query_string = None
        base_url = unicode(location)
        params["base_url"] = base_url

        output = []
        output.append(u"""<?xml version='1.0' encoding='utf-8'?>
<rss version="2.0" xmlns:atom="http://www.w3.org/2005/Atom">
    <channel>
        <title><![CDATA[%(title)s]]></title>
        <link><![CDATA[%(base_url)s]]></link>
        <description><![CDATA[%(description)s]]></description>
        <language>%(language)s</language>
        <pubDate>%(now)s</pubDate>
        <atom:link href="%(url)s" rel="self" type="application/rss+xml" />
""" % params)

        if feed.image:
            image_uri = feed.image.uri

            if image_uri.startswith("/"):
                image_uri = base_url + image_uri[1:]

            params["image_url"] = image_uri
            output.append(u"""
        <image>
            <url>%(image_url)s</url>
            <title>%(title)s</title>
            <link>%(base_url)s</link>
        </image>""" % params)

        if feed.ttl:
            output.append(u"""
        <ttl>%d</ttl>""" % feed.ttl)

        context = {
            "__builtins__": __builtin__,
            "feed": feed,
            "language": get_language(),
            "cms": cms,
            "translations": translations
        }

        items = feed.select_items()
        items.add_filter(PermissionExpression(
            get_current_user(),
            ReadPermission)
        )

        # Compile the expressions for item contents
        item_title_code = compile(
            feed.item_title_expression,
            "Feed #%d item_title_expression" % feed.id,
            "eval"
        )

        item_link_code = compile(
            feed.item_link_expression,
            "Feed #%d item_link_expression" % feed.id,
            "eval"
        )

        item_publication_date_code = compile(
            feed.item_publication_date_expression,
            "Feed #%d item_publication_date_expression" % feed.id,
            "eval"
        )

        item_description_code = compile(
            feed.item_description_expression,
            "Feed #%d item_description_expression" % feed.id,
            "eval"
        )

        for item in items:
            context["item"] = item

            title = eval(item_title_code, context)
            if not title:
                continue

            link = eval(item_link_code, context)
            if not link:
                continue

            if link.startswith("/"):
                link = base_url + link[1:]

            output.append(
u"""        <item>
            <title><![CDATA[%(title)s]]></title>
            <url><![CDATA[%(url)s]]></url>
            <guid isPermaLink="true"><![CDATA[%(url)s]]></guid>
        """ % {
                "title": title,
                "url": link
            })

            pub_date = eval(item_publication_date_code, context)
            if pub_date:
                output.append(
u"""            <pubDate><![CDATA[%s]]></pubDate>""" % rfc822_date(pub_date))

            description = eval(item_description_code, context)
            if description:
                output.append(
u"""            <description><![CDATA[%s]]></description>""" % description)

            output.append(
u"""        </item>""")

        output.append(u"""
    </channel>
</rss>""")
        return u"\n".join(output).encode("utf-8")




#-*- coding: utf-8 -*-
"""

@author:		Marc Pérez
@contact:		marc.perez@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
from cocktail.modeling import ListWrapper
from cocktail.translations import translations, get_language
from cocktail import schema
from woost.models import Extension, Language

translations.define("GoogleSearchExtension",
    ca = u"Cercador de Google",
    es = u"Buscador de Google",
    en = u"Google search"
)

translations.define("GoogleSearchExtension-plural",
    ca = u"Cercadors de Google",
    es = u"Buscadores de Google",
    en = u"Google search"
)

translations.define("GoogleSearchExtension.search_engine_id",
    ca = u"Identificador del motor de cerca",
    es = u"Indentificador del motor de busqueda",
    en = u"Search engine id"
)

translations.define("GoogleSearchExtension.results_per_page",
    ca = u"Número de resultats per pàgina",
    es = u"Número de resultados por página",
    en = u"Results number per page"
)

class GoogleSearchExtension(Extension):

    url_template = (
        "http://www.google.com/cse"
        "?cx=%(search_engine_id)s"
        "&query=%(query)s"
        "&start=%(page)d"
        "&num=%(page_size)d"
        "&lr=%(language)s"
        "&filter=%(filter)d"
        "&client=google-csbe"
        "&output=xml_no_dtd"
        "&ie=utf8"
        "&oe=utf8"
    )

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Proporciona els elements essencials per implementar el cercador
            de Google""",
            "ca"
        )
        self.set("description",
            u"""Proporciona los elementos esenciales para implementar el
            buscador de Google""",
            "es"
        )
        self.set("description",
            u"""Provides the essential elements to set up Google Search""",
            "en"
        )

    search_engine_id = schema.String(
        required = True,
        text_search = False
    )

    results_per_page = schema.Integer(
        min = 1,
        max = 20
    )

    def _load(self):
        from woost.extensions.googlesearch import strings

    def search(self,
        query,
        page = 0,
        page_size = 10,
        language = None,
        filter = True):
        """Returns Google CSE results for the given query.

        @param query: The query to search.
        @type query: unicode

        @param page: The ordinal index of the results page to return, starting
            at 0.
        @type page: int

        @param page_size: The maximum number of results per page.
        @type page_size: int

        @param language: Restricts search results to matches for the given
            language. The language must be indicated using a two letter ISO
            language code.
        @type language: str

        @param filter: Indicates if Google should apply filtering over the
            search results (ie. to remove redundant matches).
        @type filter: bool

        @return: The results for the given query.
        @rtype: L{GoogleSearchResultsList}
        """
        from xml.sax import parseString
        from xml.sax.handler import ContentHandler
        from urllib import urlopen, quote_plus

        page_size = self.results_per_page or 10

        cse_url = self.url_template % {
            "search_engine_id": self.search_engine_id,
            "query": quote_plus(query.encode("utf-8")),
            "page": page * page_size,
            "page_size": page_size,
            "language": language or get_language(),
            "filter": int(bool(filter))
        }

        f = urlopen(cse_url)
        xml_response = f.read()
        f.close()

        results = []

        READING_TOTAL_COUNT = 1
        READING_TITLE = 2
        READING_URL = 3
        READING_CONTEXT = 4

        class SAXContentHandler(ContentHandler):
            result_count = 0
            current_result = None
            status = None

            def startElement(self, name, attrs):
                if name == "M":
                    self.status = READING_TOTAL_COUNT
                elif name == "T":
                    self.status = READING_TITLE
                elif name == "U":
                    self.status = READING_URL
                elif name == "S":
                    self.status = READING_CONTEXT
                elif name == "R":
                    result = GoogleSearchResult()
                    results.append(result)
                    self.current_result = result

            def endElement(self, name):
                self.status = None
                if name == "R":
                    self.current_result = None

            def characters(self, content):
                if self.status == READING_TOTAL_COUNT:
                    self.result_count = int(content)
                elif self.status == READING_TITLE:
                    self.current_result.title += content
                elif self.status == READING_URL:
                    self.current_result.url += content
                elif self.status == READING_CONTEXT:
                    self.current_result.context += content
                    self.current_result.context = self.current_result.context.replace('<br>','')

        sax_handler = SAXContentHandler()
        parseString(xml_response, sax_handler)

        return GoogleSearchResultsList(
            results,
            page,
            page_size,
            sax_handler.result_count
        )


class GoogleSearchResult(object):
    title = ""
    url = ""
    context = ""


class GoogleSearchResultsList(ListWrapper):

    def __init__(self, results, page, page_size, result_count):
        ListWrapper.__init__(self, results)
        self.page = page
        self.page_size = page_size
        self.result_count = result_count




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from threading import Lock
import Image
from cocktail.modeling import ListWrapper
from woost.models.rendering.effectschain import (
    apply_effects_chain,
    normalize_effects_chain
)


class ContentRenderersRegistry(ListWrapper):

    def __init__(self, items = None):
        ListWrapper.__init__(self, items)
        self.__lock = Lock()

    def register(self, renderer, after = None, before = None):
        """Add a new content renderer to the registry.

        By default, renderers are appended at the end of the registry. The
        L{after} or L{before} parameters can be used to specify the position in
        the registry where the renderer should be inserted. This can be
        important, since renderer are tried in registration order

        @param renderer: The renderer to register.
        @type renderer: L{ContentRenderer}
        """
        with self.__lock:

            marker = after or before

            if marker is None:
                self._items.append(renderer)
            else:
                if after and before:
                    raise ValueError(
                        "Can't register content renderer '%s' specifying both "
                        "'after' and 'before' parameters"
                        % renderer
                    )

                # Find the requested position
                if isinstance(marker, type):

                    # Find the first instance of the given type
                    for pos, registered_renderer in enumerate(self._items):
                        if isinstance(registered_renderer, marker):
                            break
                    else:
                        pos = -1
                else:
                    # Find an specific instance
                    try:
                        pos = self._items.index(marker)
                    except IndexError:
                        pos = -1

                if pos == -1:
                    self._items.append(renderer)
                elif before:
                    self._items.insert(pos, renderer)
                else:
                    self._items.insert(pos + 1, renderer)

    def render(self,
        item,
        effects = None,
        apply_item_effects = True,
        **kwargs):

        # Composite the effects defined by the rendered item with the
        # additional effects required for the current rendering operation
        if apply_item_effects and not isinstance(item, type):
            item_effects = getattr(item, "image_effects", None)
            if item_effects:
                effects = (
                    normalize_effects_chain(item_effects)
                  + normalize_effects_chain(effects)
                )

        # Look for the first renderer that can handle the item
        for renderer in self:
            if renderer.enabled and renderer.can_render(item):
                image = renderer.render(item, **kwargs)

                # Apply rendering effects
                if effects:
                    if isinstance(image, basestring):
                        image = Image.open(image)
                    image = apply_effects_chain(image, effects)

                return image


content_renderers = ContentRenderersRegistry()
icon_renderers = ContentRenderersRegistry()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from tpv.paymentgateway import PaymentGateway
from cocktail.translations import translations
from cocktail import schema
from cocktail.controllers import context
from woost.models import Document
from woost.extensions.payments.paymentgateway \
    import PaymentGateway as CMSPaymentGateway


class DummyPaymentGateway(CMSPaymentGateway, PaymentGateway):
    """A simulated payment gateway, useful for testing purposes."""

    instantiable = True
    members_order = [
        "payment_status",
        "payment_successful_page",
        "payment_failed_page"
    ]

    payment_status = schema.String(
        required = True,
        enumeration = ("accepted", "failed"),
        default = "accepted",
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(
                "woost.extensions.payments."
                "DummyPaymentGateway.payment_status " + value,
                language,
                **kwargs
            ),
        text_search = False
    )

    payment_successful_page = schema.Reference(
        type = Document,
        related_end = schema.Reference()
    )

    payment_failed_page = schema.Reference(
        type = Document,
        related_end = schema.Reference()
    )

    def get_payment_form_data(self, payment_id, language = None):
        return (
            context["cms"].application_uri("dummy_payment_gateway"),
            (
                ("gateway_id", self.id),
                ("payment_id", payment_id)
            )
        )

    def process_notification(self, parameters):
        payment_id = parameters["payment_id"]
        payment = self.get_payment(payment_id)
        payment.status = self.payment_status
        return payment




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
import cherrypy
from cocktail.persistence import datastore
from cocktail.controllers import session
from woost.extensions.shop.shoporder import ShopOrder
from woost.extensions.shop.shoporderentry import ShopOrderEntry
from woost.extensions.shop.product import Product


class Basket(object):

    session_key = "woost.extensions.shop basket"

    @classmethod
    def get(cls):
        """Returns the shop order for the current user session.

        If the user had not started an order yet, a new one is created.

        @rtype: L{ShopOrder<woost.extensions.shop.shoporder.ShopOrder>}
        """
        order = getattr(cherrypy.request, "woost_shop_basket", None)

        if order is None:
            order = cls.restore()

            if order is None:
                order = ShopOrder()

            cherrypy.request.woost_shop_basket = order

        return order

    @classmethod
    def drop(cls):
        """Drops the current shop order."""
        session.pop(cls.session_key, None)

    @classmethod
    def store(cls):
        order = cls.get()
        order.insert()
        datastore.commit()
        session[cls.session_key] = order.id

    @classmethod
    def restore(cls):
        session_data = session.get(cls.session_key)

        if session_data is None:
            return None
        else:
            return ShopOrder.get_instance(session_data)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2009
"""
from __future__ import with_statement
from cocktail.translations import set_language
from woost.tests.models.basetestcase import BaseTestCase


class IsAccessibleExpressionTestCase(BaseTestCase):

    def setUp(self):
        from woost.models import User
        BaseTestCase.setUp(self)
        set_language("en")
        self.user = User()
        self.user.insert()

    def accessible_items(self):
        from woost.models import Publishable, IsAccessibleExpression
        return set(Publishable.select(IsAccessibleExpression(self.user)))

    def test_enabled(self):

        from woost.models import Publishable, ReadPermission

        self.everybody_role.permissions.append(
            ReadPermission(
                matching_items = {
                    "type": "woost.models.publishable.Publishable"
                }
            )
        )

        a = Publishable()
        a.enabled = True
        a.insert()

        b = Publishable()
        b.enabled = False
        b.insert()

        c = Publishable()
        c.enabled = True
        c.insert()

        d = Publishable()
        d.enabled = False
        d.insert()

        assert self.accessible_items() == set([a, c])

    def test_translation_enabled(self):

        from cocktail.translations import language_context
        from woost.models import (
            Publishable,
            Language,
            ReadPermission,
            ReadTranslationPermission
        )

        self.everybody_role.permissions.append(
            ReadPermission(
                matching_items = {
                    "type": "woost.models.publishable.Publishable"
                }
            )
        )

        self.everybody_role.permissions.append(ReadTranslationPermission())

        site_language = Language()
        site_language.iso_code = "en"
        site_language.insert()

        with language_context("en"):
            a = Publishable()
            a.per_language_publication = True
            a.translation_enabled = True
            a.insert()

            b = Publishable()
            b.per_language_publication = True
            b.translation_enabled = False
            b.insert()

            c = Publishable()
            c.per_language_publication = True
            c.translation_enabled = True
            c.insert()

            d = Publishable()
            d.per_language_publication = True
            d.set("translation_enabled", True, "de")
            d.insert()

            assert self.accessible_items() == set([a, c])

            site_language.enabled = False
            assert not self.accessible_items()

    def test_current(self):

        from woost.models import Publishable, ReadPermission
        from datetime import datetime, timedelta

        self.everybody_role.permissions.append(
            ReadPermission(
                matching_items = {
                    "type": "woost.models.publishable.Publishable"
                }
            )
        )

        now = datetime.now()

        a = Publishable()
        a.enabled = True
        a.insert()

        b = Publishable()
        b.enabled = True
        b.start_date = now
        b.end_date = now + timedelta(days = 1)
        b.insert()

        c = Publishable()
        c.enabled = True
        c.start_date = now + timedelta(days = 1)
        c.insert()

        d = Publishable()
        d.enabled = True
        d.end_date = now - timedelta(days = 1)
        d.insert()

        assert self.accessible_items() == set([a, b])

    def test_allowed(self):

        from woost.models import Publishable, ReadPermission

        a = Publishable()
        a.enabled = True
        a.insert()

        b = Publishable()
        b.enabled = True
        b.insert()

        self.everybody_role.permissions.append(
            ReadPermission(
                matching_items = {
                    "type": "woost.models.publishable.Publishable",
                    "filter": "member-id",
                    "filter_operator0": "ne",
                    "filter_value0": str(b.id)
                }
            )
        )

        assert self.accessible_items() == set([a])




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
from datetime import datetime
from cStringIO import StringIO
import cherrypy
from cocktail.modeling import cached_getter
from cocktail.events import when
from cocktail.translations import translations
from cocktail import schema
from cocktail.controllers.formcontrollermixin import FormControllerMixin
from cocktail.persistence import datastore
from woost.models import (
    Publishable,
    Language,
    get_current_user
)
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from woost.extensions.staticsite.staticsitedestination import StatusTracker
from woost.extensions.staticsite.exportationpermission import \
    ExportationPermission


class ExportStaticSiteController(
    FormControllerMixin,
    BaseBackOfficeController
):
    view_class = "woost.extensions.staticsite.ExportStaticSiteView"
    selection = None

    def __init__(self, *args, **kwargs):
        BaseBackOfficeController.__init__(self, *args, **kwargs)
        FormControllerMixin.__init__(self, *args, **kwargs)
        self.selection = self.params.read(
            schema.Collection("selection", items = schema.Reference(type = Publishable))
        )


    @cached_getter
    def form_model(self):

        from woost.extensions.staticsite import StaticSiteExtension
        extension = StaticSiteExtension.instance

        site_languages = Language.codes

        def allowed_destinations():
            return [
                destination
                for destination in extension.destinations
                if get_current_user().has_permission(
                    ExportationPermission,
                    destination = destination
                )
            ]

        return schema.Schema("ExportStaticSite", members = [
            schema.Reference(
                "snapshoter",
                type =
                    "woost.extensions.staticsite.staticsitesnapshoter."
                    "StaticSiteSnapShoter",
                required = True,
                enumeration = lambda ctx: extension.snapshoters,
                edit_control =
                    "cocktail.html.RadioSelector"
                    if len(allowed_destinations()) > 1
                    else "cocktail.html.HiddenInput",
                default = schema.DynamicDefault(lambda: extension.snapshoters[0])
            ),
            schema.Reference(
                "destination",
                type =
                    "woost.extensions.staticsite.staticsitedestination."
                    "StaticSiteDestination",
                required = True,
                enumeration = lambda ctx: allowed_destinations(),
                edit_control =
                    "cocktail.html.RadioSelector"
                    if len(allowed_destinations()) > 1
                    else "cocktail.html.HiddenInput",
                default = schema.DynamicDefault(lambda: allowed_destinations()[0])
            ),
            schema.Boolean(
                "update_only",
                required = True,
                default = True
            ),
            schema.Boolean(
                "follow_links",
                required = True,
                default = True
            )
        ])

    def submit(self):
        FormControllerMixin.submit(self)

        export_events = []
        tracker = StatusTracker()

        @when(tracker.file_processed)
        def handle_file_processed(event):
            if event.status == "failed":
                event.error_handled = True
            export_events.append(event)

        form = self.form_data

        user = get_current_user()
        user.require_permission(
            ExportationPermission,
            destination = form["destination"]
        )

        destination = form["destination"]
        snapshoter = form["snapshoter"]

        context = self.form_data.copy()

        exporter_context = destination.export(
            snapshoter,
            self.selection,
            update_only = form["update_only"],
            status_tracker = tracker,
            context = context
        )

        # View class
        self.view_class = destination.view_class(exporter_context)

        self.output["export_events"] = export_events
        self.output.update(
            **destination.output(exporter_context)
        )

        datastore.commit()

    @cached_getter
    def output(self):
        output = BaseBackOfficeController.output(self)
        output.update(
            selection = self.selection
        )
        return output




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from __future__ import with_statement
from itertools import chain
from urllib import urlencode
import cherrypy
from cocktail.modeling import getter, cached_getter
from cocktail.iteration import first
from cocktail.translations import translations, get_language
from cocktail.events import event_handler
from cocktail import schema
from cocktail.controllers import get_parameter, CookieParameterSource
from woost.models import Item
from woost.controllers import BaseCMSController
from woost.controllers.notifications import notify_user
from woost.controllers.backoffice.useractions import (
    get_user_action,
    SelectionError
)
from woost.controllers.backoffice.editstack import (
    EditNode,
    SelectionNode,
    RelationNode,
    WrongEditStackError
)


class BaseBackOfficeController(BaseCMSController):
    """Base class for all backoffice controllers."""

    section = None
    settings_duration = 60 * 60 * 24 * 30 # ~= 1 month

    @cached_getter
    def visible_languages(self):
        return get_parameter(
            schema.Collection(
                "language",
                items = schema.String(),
                default = [get_language()]
            ),
            source = CookieParameterSource(
                cookie_naming = "visible_languages",
                cookie_duration = self.settings_duration
            )
        )

    # URIs and navigation
    #--------------------------------------------------------------------------
    def edit_uri(self, target, *args, **kwargs):
        """Get the URI of the edit page of the specified item.

        @param target: The item or content type to get the URI for.
        @type target: L{Item<woost.models.Item>} instance or class

        @param args: Additional path components that will be appended to the
            produced URI.
        @type args: unicode

        @param kwargs: Additional query string parameters that will be added
            to the produced URI.

        @return: The produced URI.
        @rtype: unicode
        """
        params = kwargs or {}
        edit_stack = self.edit_stack

        if edit_stack and "edit_stack" not in params:
            params["edit_stack"] = edit_stack.to_param()

        # URI for new items
        if isinstance(target, type) or not target.is_inserted:
            target_id = "new"
            if edit_stack is None \
            or ("edit_stack" in params and params["edit_stack"] is None):
                params["item_type"] = target.full_name

        # URI for existing items
        else:
            primary_member = target.__class__.primary_member

            if primary_member is None:
                raise TypeError("Can't edit types without a primary member")

            target_id = target.get(primary_member)

            if target_id is None:
                raise ValueError("Can't edit objects without an identifier")

        uri = self.contextual_uri(
            "content",
            target_id,
            *(args or ["fields"])
        )

        if params:
            uri += "?" + urlencode(
                dict((k, v) for k, v in params.iteritems() if v is not None),
                True
            )

        return uri

    def go_back(self):
        """Redirects the user to its previous significant location."""

        edit_stack = self.edit_stack

        # Go back to the parent edit state
        if edit_stack and len(edit_stack) > 1:
            if isinstance(edit_stack[-2], RelationNode):
                edit_stack.go(-3)
            else:
                edit_stack.go(-2)

        # Go back to the root of the backoffice
        else:
            raise cherrypy.HTTPRedirect(
                edit_stack and edit_stack.root_url or self.contextual_uri()
            )

    # Edit stack
    #--------------------------------------------------------------------------
    @getter
    def edit_stack(self):
        """The edit stack for the current request.
        @type: L{EditStack<woost.controllers.backoffice.editstack.EditStack>}
        """
        return self.context["edit_stacks_manager"].current_edit_stack

    @getter
    def stack_node(self):
        """The top node of the edit stack for the current request.
        @type: L{StackNode<woost.controllers.backoffice.editstack.StackNode>}
        """
        stack = self.edit_stack
        return stack and stack[-1]

    @getter
    def edit_node(self):
        """The topmost edit node of the edit stack for the current request.
        @type: L{EditNode<woost.controllers.backoffice.editstack.EditNode>}
        """
        stack = self.edit_stack
        if stack:
            return stack[-1].get_ancestor_node(EditNode, include_self = True)
        return None

    @getter
    def relation_node(self):
        """The topmost relation node of the edit stack for the current request.
        @type: L{RelationNode<woost.controllers.backoffice.editstack.RelationNode>}
        """
        stack = self.edit_stack
        if stack:
            return stack[-1].get_ancestor_node(
                RelationNode,
                include_self = True
            )
        return None

    # Request flow
    #--------------------------------------------------------------------------
    @event_handler
    def handle_exception_raised(cls, event):

        # Redirect the user to the backoffice root when failing to load an edit
        # stack node
        if isinstance(
            event.exception,
            WrongEditStackError
        ):
            notify_user(translations(event.exception), "error")
            raise cherrypy.HTTPRedirect(event.source.contextual_uri())

    def _invoke_user_action(self, action, selection):
        for error in action.get_errors(self, selection):
            self._handle_user_action_error(action, selection, error)

        action.invoke(self, selection)

    _graceful_user_action_errors = set([SelectionError])

    def _handle_user_action_error(self, action, selection, error):
        if isinstance(error, tuple(self._graceful_user_action_errors)):
            notify_user(translations(error), "error")
            self.go_back()
        else:
            raise error

    def _get_user_action(self, param_key = "action"):
        action = None
        action_id = self.params.read(schema.String(param_key))

        if action_id:
            action = get_user_action(action_id)
            if action and not action.enabled:
                action = None

        return action

    @cached_getter
    def client_side_scripting(self):
        return get_parameter(schema.Boolean("client_side_scripting"))

    @cached_getter
    def output(self):
        output = BaseCMSController.output(self)
        output.update(
            backoffice = self.context["publishable"],
            section = self.section,
            edit_stack = self.edit_stack,
            edit_uri = self.edit_uri,
            client_side_scripting = self.client_side_scripting
        )
        return output


class EditStateLostError(Exception):
    """An exception raised when a user requests an edit state that is no longer
    available."""




#-*- coding: utf-8 -*-
"""
Classes for choosing icons for CMS items.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			April 2009
"""
import os
from pkg_resources import resource_filename
from cocktail.typemapping import TypeMapping
from cocktail.cache import Cache
from woost.models.item import Item
from woost.models.publishable import Publishable
from woost.models.file import File


class IconResolver(object):
    """An object that chooses icons for CMS items.

    @ivar icon_format: The image format that icons are stored in. Takes the
        same values accepted by PIL images.
    @type icon_format: str

    @ivar icon_repositories: A list of the directories to search for icons, in
        descending order of preference.
    @type icon_repositories: str list

    @ivar file_resolvers: A mapping of types and icon resolution functions.
        This allows to override the icon resolution machinery for certain
        content types. Resolution functions should take as parameters a content
        type and (optionally) an item, and return a list of possible icon file
        names that would be a match for them.
    @type file_resolvers: (L{Item<woost.models.Item>} class, callable)
        L{TypeMapping<cocktail.typemapping.TypeMapping>}

    @ivar name_resolution_cache: A cache that stores the result of the
        L{find_icon} method given a list of possible names produced by the
        L{file_resolvers} mapping.
    @type: L{Cache<cocktail.cache.Cache>}
    """
    icon_format = "png"
    _icon_extension = None
    icon_repositories = []
    file_resolvers = None
    name_resolution_cache = None

    def __init__(self):
        self.icon_repositories = [
            resource_filename("woost.views", "resources/images/icons")
        ]
        self.file_resolvers = TypeMapping()
        self.file_resolvers[Item] = self._resolve_item
        self.file_resolvers[Publishable] = self._resolve_publishable
        self.file_resolvers[File] = self._resolve_file
        self.name_resolution_cache = Cache(load = self._find_icon)

    def _get_icon_extension(self):
        return self._icon_extension or self.icon_format

    def _set_icon_extension(self, extension):
        self._icon_extension = extension

    icon_extension = property(_get_icon_extension, _set_icon_extension,
        doc = """The file extension for icon files. If a explicit value is not
        provided, it will be infered from L{icon_format}.
        @type: str
        """)

    def find_icon(self, item, size):
        """Obtains the path to the icon file that best represents the indicated
        CMS item or content type.

        The default implementation searches for icons in the directories
        specified by the L{icon_repositories} property, in order, returning the
        first icon whose name matches the name of one of the item's classes.
        Also, publishable items and files are special cased, to take their
        resource and MIME types into account.

        @param item: The item or content type to obtain the icon for.
        @type item: L{Item<woost.models.Item>} instance or class

        @param size: The size of the icon to look for, in pixels.
        @type size: int

        @return: The path to the best matching icon, or None if no matching
            icon is found.
        @rtype: str
        """
        if isinstance(item, type):
            content_type = item
            item = None
        else:
            content_type = item.__class__

        file_resolver = self.file_resolvers.get(content_type)

        if file_resolver is not None:
            file_names = file_resolver(content_type, item)
            if file_names:
                key = (size, tuple(file_names))
                return self.name_resolution_cache.request(key)

        return None

    def _find_icon(self, key):

        size, file_names = key

        repositories = self.icon_repositories
        size_str = "%dx%d" % (size, size)
        icon_ext = "." + self.icon_extension

        for repository in repositories:
            for file_name in file_names:
                icon_path = os.path.join(
                    repository,
                    size_str,
                    file_name + icon_ext
                )
                if os.path.exists(icon_path):
                    return icon_path

        return None

    def _resolve_item(self, content_type, item):
        return [
            "type-" + cls.full_name.replace(".", "-")
            for cls in content_type.ascend_inheritance(True)
        ]

    def _resolve_publishable(self, content_type, item):
        file_names = self._resolve_item(content_type, item)

        if item is not None and item.resource_type:
            file_names.insert(0, "resource-type-" + item.resource_type)

        return file_names

    def _resolve_file(self, content_type, item):
        file_names = self._resolve_publishable(content_type, item)

        if item is not None and item.mime_type:
            mime = item.mime_type.replace("/", "-").replace(".", "-")
            file_names.insert(0, "mime-" + mime)

        return file_names




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from cocktail.html import Element, templates
from woost.models import Item

ContentTypeTree = \
    templates.get_class("woost.views.ContentTypeTree")


class ContentTypeSelector(Element):

    value = None
    Tree = ContentTypeTree
    root = Item
    root_visible = True

    def _build(self):
        self.add_class("selector")

    def _ready(self):

        self.tree = self.create_tree()
        self.label = self.create_label()

        self.append(self.label)
        self.append(self.tree)

        Element._ready(self)

    def create_label(self):
        label = self.tree.create_label(self.value)
        label.tag = "span"
        label.add_class("label")
        return label

    def create_tree(self):
        tree = self.Tree()
        tree.add_class("selector_content")
        tree.root = self.root
        tree.root_visible = self.root_visible
        return tree




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from cocktail.html import Element, templates
from woost.models import Publishable, File
from woost.models.rendering import ImageFactoryMember
from woost.extensions.blocks.block import Block


class TranslatedRichTextBlock(Block):

    instantiable = True
    view_class = "cocktail.html.Element"

    members_order = ["text", "image", "image_factory", "attachments"]

    text = schema.String(
        required = True,
        edit_control = "woost.views.RichTextEditor",
        translated = True,
        member_group = "content"
    )

    image = schema.Reference(
        type = File,
        related_end = schema.Collection(),
        relation_constraints = [File.resource_type.equal("image")],
        member_group = "content"
    )

    image_factory = ImageFactoryMember(
        member_group = "content"
    )

    attachments = schema.Collection(
        items = schema.Reference(type = Publishable),
        related_end = schema.Collection(),
        member_group = "content"
    )

    def init_view(self, view):

        Block.init_view(self, view)

        if self.image is None:
            view.append(self.text)
        else:
            view.add_class("text_with_image")

            image = templates.new("woost.views.Image")
            image.image = self.image
            image.image_factory = self.image_factory
            view.append(image)

            text_container = Element()
            text_container.append(self.text)
            view.append(text_container)




#-*- coding: utf-8 -*-
u"""Defines the `FacebookPublicationExtension` class.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from woost.models import Extension


translations.define("FacebookPublicationExtension",
    ca = u"Publicació a Facebook",
    es = u"Publicación en Facebook",
    en = u"Publish to Facebook"
)

translations.define("FacebookPublicationExtension-plural",
    ca = u"Publicació a Facebook",
    es = u"Publicación en Facebook",
    en = u"Publish to Facebook"
)


class FacebookPublicationExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Simplifica la publicació de continguts del lloc web al Facebook""",
            "ca"
        )
        self.set("description",
            u"""Simplifica la publicación de contenidos del sitio web en
            Facebook""",
            "es"
        )
        self.set("description",
            u"""Makes it easy to publish content from the website to Facebook""",
            "en"
        )

    def _load(self):

        from woost.extensions.facebookpublication import (
            strings,
            fbpublishuseraction,
            fbpublishauthuseraction,
            facebookpublicationpermission
        )

        from woost.extensions.facebookpublication.facebookpublicationtarget \
            import FacebookPublicationTarget

        FacebookPublicationExtension.add_member(
            schema.Collection("targets",
                items = schema.Reference(type = FacebookPublicationTarget),
                integral = True,
                bidirectional = True,
                related_end = schema.Collection()
            )
        )

        from woost.controllers.backoffice.backofficecontroller \
            import BackOfficeController

        from woost.extensions.facebookpublication.facebookpublicationcontroller \
            import FacebookPublicationController

        BackOfficeController.fbpublish = FacebookPublicationController

        from woost.extensions.facebookpublication.facebookalbumscontroller \
            import FacebookAlbumsController

        BackOfficeController.fbpublish_albums = FacebookAlbumsController




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost import app
from woost.models.rendering.contentrenderer import ContentRenderer
from woost.models.rendering.contentrenderersregistry import icon_renderers


class IconRenderer(ContentRenderer):

    def can_render(self, item):
        return True

    def render(self, item, size = 16):
        return app.icon_resolver.find_icon(item, size)


icon_renderers.register(IconRenderer())




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os

if hasattr(os, "symlink"):
    from cocktail.styled import styled
    from cocktail.events import when
    from cocktail.persistence import datastore
    from woost import app
    from woost.models.publishable import Publishable
    from woost.models.site import Site
    from woost.models.file import File
    from woost.models.user import User
    from woost.models.rendering.cache import require_rendering

    debug = True
    filesystem_encoding = "utf-8"

    members_affecting_static_publication = set([
        File.title,
        File.file_name,
        File.image_effects,
        Publishable.enabled,
        Publishable.start_date,
        Publishable.end_date
    ])

    @when(File.inserted)
    @when(File.changed)
    @when(File.deleting)
    def _schedule_links_update(e):

        file = e.source
        member = getattr(e, "member", None)

        if member is not None:

            if not file.is_inserted:
                return

            if member not in members_affecting_static_publication:
                return

        # Store the current links, to be removed when the current transaction is
        # committed
        key = "woost.models.File.old_links-%d" % file.id
        old_links = datastore.get_transaction_value(key)

        if old_links is None:
            old_links = set()
            datastore.set_transaction_value(key, old_links)

        old_links.update(get_links(file))

        # Add an after commit hook to remove/create the file's links
        datastore.unique_after_commit_hook(
            "woost.models.File.update_static_links-%d" % file.id,
            _update_links_after_commit,
            file,
            old_links
        )

    def _update_links_after_commit(commit_successful, file, old_links):
        if commit_successful:
            remove_links(file, old_links)
            create_links(file)

    def encode_filename(link, encoding = None):
        if encoding is None:
            encoding = filesystem_encoding
        return link.encode(encoding) if isinstance(link, unicode) else link

    def create_links(file, links = None, encoding = None):

        if not file.is_inserted:
            return

        anonymous = User.require_instance(qname = "woost.anonymous_user")

        if not file.is_accessible(anonymous):
            return

        if links is None:
            links = get_links(file)

        if file.image_effects:
            linked_file = require_rendering(file)
        else:
            linked_file = file.file_path

        linked_file = encode_filename(linked_file, encoding)

        for link in links:

            link = encode_filename(link, encoding)

            if debug:
                print styled("STATIC PUBLICATION", "white", "green"),
                print "Adding link for #%s (%s -> %s)" % (
                    styled(file.id, style = "bold"),
                    styled(link, "yellow"),
                    styled(linked_file, "brown")
                )

            # Delete the link if it already existed
            try:
                os.remove(link)
            except OSError:
                pass

            # Create the new link
            os.symlink(linked_file, link)

    def remove_links(file, links = None, encoding = None):

        if links is None:
            links = get_links(file)

        for link in links:
            link = encode_filename(link, encoding)
            if os.path.lexists(link):
                if debug:
                    print styled("STATIC PUBLICATION", "white", "red"),
                    print "Removing link for #%s (%s)" % (
                        styled(file.id, style = "bold"),
                        styled(link, "yellow")
                    )
                os.remove(link)

    def get_links(file):
        site = Site.main
        return [
            app.path("static", site.main.get_path(file, language))
            for language in (file.translations or [None])
        ]




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from decimal import Decimal
from cocktail.translations import get_language, translations
from cocktail import schema
from woost.models import Item, Site
from woost.extensions.countries.country import Country
from woost.extensions.shop.shoporderentry import ShopOrderEntry


class ShopOrder(Item):

    members_order = [
        "address",
        "town",
        "region",
        "country",
        "postal_code",
        "cost",
        "entries"
    ]

    address = schema.String(
        member_group = "shipping_info",
        required = True,
        listed_by_default = False
    )

    town = schema.String(
        member_group = "shipping_info",
        required = True,
        listed_by_default = False
    )

    region = schema.String(
        member_group = "shipping_info",
        required = True,
        listed_by_default = False
    )

    country = schema.Reference(
        member_group = "shipping_info",
        type = Country,
        related_end = schema.Collection(),
        required = True,
        listed_by_default = False,
        user_filter = "cocktail.controllers.userfilter.MultipleChoiceFilter"
    )

    postal_code = schema.String(
        member_group = "shipping_info",
        required = True,
        listed_by_default = False
    )

    cost = schema.Decimal(
        required = True,
        default = Decimal("0"),
        editable = False
    )

    language = schema.String(
        required = True,
        format = "^[a-z]{2}$",
        editable = False,
        default = schema.DynamicDefault(get_language),
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(value, language, **kwargs)
    )

    status = schema.String(
        required = True,
        indexed = True,
        enumeration = ["pending", "accepted", "failed"],
        default = "pending",
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(
                "woost.extensions.shop.ShopOrder.status " + value,
                language,
                **kwargs
            )
    )

    entries = schema.Collection(
        items = "woost.extensions.shop.shoporderentry.ShopOrderEntry",
        integral = True,
        bidirectional = True,
        min = 1
    )

    def calculate_cost(
        self,
        include_shipping = True,
        include_taxes = True,
        include_discounts = True
    ):
        """Calculates the costs for the order.
        @rtype: dict
        """
        costs = {
            "pricing_policies": [],
            "price": {
                "cost": 0,
                "percentage": 0,
                "total": None
            },
            "shipping": 0,
            "tax": {
                "cost": 0,
                "percentage": 0
            },
            "entries": [
                {
                    "pricing_policies": [],
                    "quantity": entry.quantity,
                    "paid_quantity": entry.quantity,
                    "price": {
                        "cost": entry.product.price,
                        "percentage": 0
                    },
                    "shipping": 0,
                    "tax": {
                        "cost": 0,
                        "percentage": 0
                    }
                }
                for entry in self.entries
            ]
        }

        from woost.extensions.shop import ShopExtension
        shop_ext = ShopExtension.instance

        policies = list()

        if include_discounts:
            policies.extend(shop_ext.discounts)

        if include_shipping:
            policies.extend(shop_ext.shipping_costs)

        if include_taxes:
            policies.extend(shop_ext.taxes)

        for pricing_policy in policies:
            matching_items = pricing_policy.select_matching_items()

            if issubclass(matching_items.type, ShopOrder):
                if pricing_policy.applies_to(self):
                    pricing_policy.apply(self, costs)
                    costs["pricing_policies"].append(pricing_policy)
            else:
                for entry, entry_costs in zip(self.entries, costs["entries"]):
                    if pricing_policy.applies_to(entry.product):
                        pricing_policy.apply(entry.product, entry_costs)
                        entry_costs["pricing_policies"].append(pricing_policy)

        # Total price
        def apply_percentage(costs):
            cost = costs["cost"]
            percentage = costs["percentage"]
            if percentage:
                cost += cost * percentage / 100
            costs["total"] = cost
            return cost

        total_price = apply_percentage(costs["price"])

        for entry_costs in costs["entries"]:
            entry_price = apply_percentage(entry_costs["price"])
            entry_total_price = entry_price * entry_costs["paid_quantity"]
            entry_costs["total_price"] = entry_total_price
            total_price += entry_total_price

        costs["total_price"] = total_price

        # Total taxes
        total_taxes = costs["tax"]["cost"] \
                    + total_price * costs["tax"]["percentage"] / 100

        for entry_costs in costs["entries"]:
            quantity = entry_costs["paid_quantity"]
            entry_price = entry_costs["price"]["total"] * quantity
            entry_taxes = entry_costs["tax"]["cost"] * quantity \
                        + entry_price * entry_costs["tax"]["percentage"] / 100
            total_taxes += entry_taxes
            entry_costs["tax"]["total"] = entry_taxes

        costs["total_taxes"] = total_taxes

        # Total shipping costs
        total_shipping_costs = costs["shipping"] \
                             + sum(entry_costs["shipping"] * entry_costs["quantity"]
                                   for entry_costs in costs["entries"])
        costs["total_shipping_costs"] = total_shipping_costs

        # Grand total
        costs["total"] = total_price + total_taxes + total_shipping_costs

        return costs

    def count_items(self):
        """Gets the number of purchased product units in the order.
        @rtype: int
        """
        return sum(entry.quantity for entry in self.entries)

    def get_product_entry(self, product):
        """Gets the entry in the order for the given product.

        @param product: The product to obtain the entry for.
        @type product: L{Product<woost.extensions.shop.product.Product>}

        @return: The matching entry, or None if the order doesn't contain an
            entry for the indicated product.
        @rtype: L{ShopOrderEntry
                  <woost.extensions.shop.shoporderentry.ShopOrderEntry>}
        """
        for entry in self.entries:
            if entry.product is product:
                return entry

    def set_product_quantity(self, product, quantity):
        """Updates the quantity of ordered units for the indicated product.

        If an entry for the given product already exists, its quantity will be
        updated to the indicated value. If the indicated quantity is zero, the
        entry will be removed from the order. If no matching entry exists, a
        new entry for the product will be created with the specified amount of
        units.

        @param product: The product to set the quantity for.
        @type product: L{Product<woost.extensions.shop.product.Product>}

        @param quantity: The number of units of the product to order.
        @type quantity: int
        """
        entry = self.get_product_entry(product)

        if entry is None:
            if quantity:
                entry = ShopOrderEntry(
                    product = product,
                    quantity = quantity
                )
                self.entries.append(entry)
                if self.is_inserted:
                    entry.insert()
        else:
            if quantity:
                entry.quantity = quantity
            else:
                if entry.is_inserted:
                    entry.delete()
                else:
                    self.entries.remove(entry)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from __future__ import with_statement
from woost.tests.models.basetestcase import BaseTestCase


class TriggerMatchTestCase(BaseTestCase):

    def assert_match(self, trigger, *tests):
        test = 1
        for target, user, context, should_match in tests:
            self.assertEqual(
                trigger.match(user, target = target, verbose = True, **context),
                should_match,
                "Trigger %s %s match test %d (target=%s user=%s context=%s)"
                % (
                    trigger,
                    "should" if should_match else "should not",
                    test,
                    target,
                    user,
                    context
                )
            )
            test += 1

    def test_user(self):

        from woost.models import ContentTrigger, Item, Role, User

        r1 = Role()
        r2 = Role()
        r3 = Role(base_roles = [r2])

        u1 = User(roles = [r1])
        u2 = User(roles = [r2])
        u3 = User(roles = [r3])

        self.assert_match(
            ContentTrigger(matching_roles = [r2]),
            (Item(), u2, {}, True),
            (Item(), u3, {}, True),
            (Item(), None, {}, False),
            (Item(), u1, {}, False)
        )

    def test_target(self):

        from woost.models import (
            ContentTrigger,
            Item,
            Publishable,
            StandardPage,
            User,
            set_current_user
        )

        self.assert_match(
            ContentTrigger(matching_items = {
                "type": "woost.models.publishable.Publishable"
            }),
            (Publishable(), None, {}, True),
            (StandardPage(), None, {}, True),
            (Item(), None, {}, False),
            (ContentTrigger(), None, {}, False)
        )

        user = User()
        set_current_user(user)

        self.assert_match(
            ContentTrigger(matching_items = {
                "type": "woost.models.publishable.Publishable",
                "filter": "owned-items"
            }),
            (Publishable(), user, {}, False),
            (Publishable(owner = user), user, {}, True)
        )

    def test_modified_member(self):

        from woost.models import ModifyTrigger, Item, Document

        self.assert_match(
            ModifyTrigger(matching_members = [
                "woost.models.item.Item.owner",
                "woost.models.document.Document.title"
            ]),
            (Item(), None, {"member": Item.owner}, True),
            (Item(), None, {"member": Item.draft_source}, False),
            (Document(), None, {"member": Document.title}, True),
            (Document(), None, {"member": Document.hidden}, False)
        )

    def test_language(self):

        from woost.models import ModifyTrigger, Item

        self.assert_match(
            ModifyTrigger(matching_languages = ["en", "fr"]),
            (Item(), None, {"language": "en"}, True),
            (Item(), None, {"language": "fr"}, True),
            (Item(), None, {"language": None}, False),
            (Item(), None, {"language": "ru"}, False)
        )


class TriggerInvocationTestCase(BaseTestCase):

    def setUp(self):

        from cocktail.persistence import datastore
        from woost.models import (
            triggerresponse,
            TriggerResponse,
            User,
            set_current_user
        )

        BaseTestCase.setUp(self)

        class TestTriggerResponse(TriggerResponse):

            def __init__(self, response_log, *args, **kwargs):
                TriggerResponse.__init__(self, *args, **kwargs)
                self.response_log = response_log

            def execute(self, items, user, batch = False, **context):
                self.response_log.append({
                    "trigger": self.trigger,
                    "items": items,
                    "user": user,
                    "batch": batch,
                    "context": context
                })

        self.TestTriggerResponse = TestTriggerResponse

        # ZODB requires classes to be accessible, so TestTriggerResponse is
        # added to the same module of its base class:
        triggerresponse.TestTriggerResponse = TestTriggerResponse
        TestTriggerResponse.__module__ = TriggerResponse.__module__

        self.user = User()
        set_current_user(self.user)

        datastore.commit()

    def make_trigger(self, trigger_type, response_log = None, **kwargs):

        from cocktail.persistence import datastore
        from woost.models.trigger import set_triggers_enabled

        if response_log is None:
            new_response_log = True
            response_log = []
        else:
            new_response_log = False

        set_triggers_enabled(False)

        try:
            trigger = trigger_type(**kwargs)
            trigger.responses = [self.TestTriggerResponse(response_log)]
            trigger.insert()
            self.site.triggers.append(trigger)
            datastore.commit()
            if new_response_log:
                return trigger, response_log
            else:
                return trigger
        finally:
            set_triggers_enabled(True)


class BeforeTestCase(TriggerInvocationTestCase):

    def test_before_create(self):

        from woost.models import CreateTrigger, Item

        # Declare the trigger
        trigger, response_log = self.make_trigger(
            CreateTrigger,
            execution_point = "before",
            batch_execution = False
        )

        qname = "item1"
        item1 = Item(qname = qname)
        assert len(response_log) == 1

        id = 234
        item2 = Item(id = id)
        assert len(response_log) == 2

        response = response_log[0]
        assert response["trigger"] is trigger
        assert response["items"] == [item1]
        assert response["user"] is self.user
        assert not response["batch"]
        assert response["context"]["values"]["qname"] == qname

        response = response_log[1]
        assert response["trigger"] is trigger
        assert response["items"] == [item2]
        assert response["user"] is self.user
        assert not response["batch"]
        assert response["context"]["values"]["id"] == id

    def test_before_insert(self):

        from woost.models import InsertTrigger, Item

        # Declare the trigger
        trigger, response_log = self.make_trigger(
            InsertTrigger,
            execution_point = "before",
            batch_execution = False
        )

        # Create two items. This should trigger the response twice.
        item1 = Item()
        item1.insert()
        assert len(response_log) == 1

        item2 = Item()
        item2.insert()
        assert len(response_log) == 2

        response = response_log[0]
        assert response["trigger"] is trigger
        assert response["items"] == [item1]
        assert response["user"] is self.user
        assert not response["batch"]

        response = response_log[1]
        assert response["trigger"] is trigger
        assert response["items"] == [item2]
        assert response["user"] is self.user
        assert not response["batch"]

    def test_before_modify(self):

        from woost.models import ModifyTrigger, Item, User

        # Declare the trigger
        trigger, response_log = self.make_trigger(
            ModifyTrigger,
            execution_point = "before",
            batch_execution = False
        )

        # Create an item and initialize it. This shouldn't trigger any
        # response, since modifications happen before the item is inserted.
        item = Item()
        item.qname = "foo"
        item.insert()
        assert not response_log

        # Modify the inserted item two times. This should trigger the response
        # twice.
        item.qname = "bar"
        assert len(response_log) == 1

        item.owner = User()
        assert len(response_log) == 2

        response = response_log[0]
        assert response["trigger"] is trigger
        assert response["items"] == [item]
        assert response["context"]["member"] is Item.qname
        assert response["user"] is self.user
        assert not response["batch"]

        response = response_log[1]
        assert response["trigger"] is trigger
        assert response["items"] == [item]
        assert response["context"]["member"] is Item.owner
        assert response["user"] is self.user
        assert not response["batch"]

    def test_before_delete(self):

        from woost.models import DeleteTrigger, Item

        # Declare the trigger
        trigger, response_log = self.make_trigger(
            DeleteTrigger,
            execution_point = "before",
            batch_execution = False
        )

        # Create and insert two items
        item1 = Item()
        item1.insert()

        item2 = Item()
        item2.insert()

        # Delete the items. This should trigger the response twice.
        item1.delete()
        assert len(response_log) == 1

        item2.delete()
        assert len(response_log) == 2

        response = response_log[0]
        assert response["trigger"] is trigger
        assert response["items"] == [item1]
        assert response["user"] is self.user
        assert not response["batch"]

        response = response_log[1]
        assert response["trigger"] is trigger
        assert response["items"] == [item2]
        assert response["user"] is self.user
        assert not response["batch"]

    def test_before_create_insert_modify_delete(self):

        from woost.models import (
            Item,
            CreateTrigger,
            InsertTrigger,
            ModifyTrigger,
            DeleteTrigger
        )

        response_log = []

        create_trigger = self.make_trigger(
            CreateTrigger,
            response_log,
            execution_point = "before",
            batch_execution = False
        )

        insert_trigger = self.make_trigger(
            InsertTrigger,
            response_log,
            execution_point = "before",
            batch_execution = False
        )

        modify_trigger = self.make_trigger(
            ModifyTrigger,
            response_log,
            execution_point = "before",
            batch_execution = False
        )

        delete_trigger = self.make_trigger(
            DeleteTrigger,
            response_log,
            execution_point = "before",
            batch_execution = False
        )

        # Create and insert an item
        item = Item(qname = "foo")
        item.insert()

        assert len(response_log) == 2
        response = response_log.pop()
        assert response["trigger"] is insert_trigger
        assert response["items"] == [item]
        assert response["user"] is self.user
        assert not response["batch"]

        response = response_log.pop()
        assert response["trigger"] is create_trigger
        assert response["items"] == [item]
        assert response["user"] is self.user
        assert response["context"]["values"]["qname"] == "foo"
        assert not response["batch"]

        # Modify the item
        item.qname = "bar"

        assert len(response_log) == 1
        response = response_log.pop()
        assert response["trigger"] is modify_trigger
        assert response["items"] == [item]
        assert response["context"]["member"] is Item.qname
        assert response["user"] is self.user
        assert not response["batch"]

        # Delete the item
        item.delete()

        assert len(response_log) == 1
        response = response_log.pop()
        assert response["trigger"] is delete_trigger
        assert response["items"] == [item]
        assert response["user"] is self.user
        assert not response["batch"]


class AfterTestCase(TriggerInvocationTestCase):

    def test_after_insert(self):

        from cocktail.persistence import datastore
        from woost.models import InsertTrigger, Item

        # Declare the trigger
        trigger, response_log = self.make_trigger(
            InsertTrigger,
            execution_point = "after",
            batch_execution = False
        )

        # Insert a new item, but abort the transaction
        # (the trigger shouldn't be called)
        item = Item()
        item.insert()
        assert not response_log
        datastore.abort()
        assert not response_log

        # Insert a new item, and commit the transaction
        # (the trigger should be executed)
        item = Item()
        item.insert()
        datastore.commit()

        assert len(response_log) == 1
        response = response_log[0]
        assert response["trigger"] is trigger
        assert response["items"] == [item]
        assert response["user"] is self.user
        assert not response["batch"]

    def test_after_modify(self):

        from cocktail.persistence import datastore
        from woost.models import ModifyTrigger, Item

        # Declare the trigger
        trigger, response_log = self.make_trigger(
            ModifyTrigger,
            execution_point = "after",
            batch_execution = False
        )

        # Create an item and initialize it. This shouldn't trigger any
        # response, since modifications happen before the item is inserted.
        item = Item()
        item.qname = "foo"
        item.insert()
        datastore.commit()
        assert not response_log

        # Modify the inserted item, but abort the transaction. Again, this
        # shouldn't trigger any response.
        item.qname = "bar"
        datastore.abort()
        assert not response_log

        # Modify the inserted item and commit the transaction. This should
        # trigger the response.
        item.qname = "spam"
        datastore.commit()

        assert len(response_log) == 1
        response = response_log[0]
        assert response["trigger"] is trigger
        assert response["items"] == [item]
        assert response["context"]["member"] is Item.qname
        assert response["user"] is self.user
        assert not response["batch"]

    def test_after_delete(self):

        from cocktail.persistence import datastore
        from woost.models import DeleteTrigger, Item

        # Declare the trigger
        trigger, response_log = self.make_trigger(
            DeleteTrigger,
            execution_point = "after",
            batch_execution = False
        )

        # Create and insert an item
        item = Item()
        item.insert()
        datastore.commit()

        # Delete the item, but abort the transaction. This shouldn't trigger
        # the response.
        item.delete()
        datastore.abort()
        assert not response_log

        # Delete the inserted item and commit the transaction. This should
        # trigger the response.
        item.delete()
        datastore.commit()

        assert len(response_log) == 1
        response = response_log[0]
        assert response["trigger"] is trigger
        assert response["items"] == [item]
        assert response["user"] is self.user
        assert not response["batch"]

    def test_after_insert_modify_delete(self):

        from cocktail.persistence import datastore
        from woost.models import (
            Item,
            User,
            InsertTrigger,
            ModifyTrigger,
            DeleteTrigger
        )

        response_log = []

        insert_trigger = self.make_trigger(
            InsertTrigger,
            response_log,
            execution_point = "after",
            batch_execution = False
        )

        modify_trigger = self.make_trigger(
            ModifyTrigger,
            response_log,
            execution_point = "after",
            batch_execution = False
        )

        delete_trigger = self.make_trigger(
            DeleteTrigger,
            response_log,
            execution_point = "after",
            batch_execution = False
        )

        # Create and insert an item
        item = Item()
        item.qname = "foo"
        item.insert()

        # Modify it
        item.qname = "bar"
        item.owner = User()

        # Delete it
        item.delete()

        # Commit the transaction; this should execute all the scheduled
        # responses
        datastore.commit()
        assert len(response_log) == 4

        response = response_log.pop(0)
        assert response["trigger"] is insert_trigger
        assert response["items"] == [item]
        assert response["user"] is self.user
        assert not response["batch"]

        for member in (Item.qname, Item.owner):
            response = response_log.pop(0)
            assert response["trigger"] is modify_trigger
            assert response["items"] == [item]
            assert response["context"]["member"] is member
            assert response["user"] is self.user
            assert not response["batch"]

        response = response_log[0]
        assert response["trigger"] is delete_trigger
        assert response["items"] == [item]
        assert response["user"] is self.user
        assert not response["batch"]


class BatchTestCase(TriggerInvocationTestCase):

    def test_after_insert_batched(self):

        from cocktail.persistence import datastore
        from woost.models import InsertTrigger, Item

        # Declare the trigger
        trigger, response_log = self.make_trigger(
            InsertTrigger,
            execution_point = "after",
            batch_execution = True
        )

        # Insert new items, but abort the transaction
        # (the trigger shouldn't be called)
        item1 = Item()
        item1.insert()
        assert not response_log

        item2 = Item()
        item2.insert()
        assert not response_log

        datastore.abort()
        assert not response_log

        # Create and insert two items, and commit the transaction. The response
        # should be triggered just once.
        item1 = Item()
        item1.insert()
        item2 = Item()
        item2.insert()
        datastore.commit()

        assert len(response_log) == 1

        response = response_log[0]
        assert response["trigger"] is trigger
        assert response["items"] == set([item1, item2])
        assert response["user"] is self.user
        assert response["batch"]

    def test_after_modify_batched(self):

        from cocktail.persistence import datastore
        from woost.models import ModifyTrigger, Item, User

        # Declare the trigger
        trigger, response_log = self.make_trigger(
            ModifyTrigger,
            execution_point = "after",
            batch_execution = True
        )

        # Create two items and initialize them. This shouldn't trigger any
        # response, since modifications happen before items are inserted.
        item1 = Item()
        item1.qname = "foo"
        item1.insert()
        item2 = Item()
        item2.owner = User()
        item2.insert()
        datastore.commit()
        assert not response_log

        # Modify the inserted items, but abort the transaction. Again, this
        # shouldn't trigger any response.
        item1.qname = "bar"
        item2.owner = User()
        datastore.abort()
        assert not response_log

        # Modify the inserted items and commit the transaction. This should
        # trigger the response just once.
        item1.qname = "spam"
        item2.owner = User()
        datastore.commit()

        assert len(response_log) == 1
        response = response_log[0]
        assert response["trigger"] is trigger
        assert response["items"] == set([item1, item2])
        assert response["context"]["modified_members"] == {
            item1: set([(Item.qname, None)]),
            item2: set([(Item.owner, None)])
        }
        assert response["user"] is self.user
        assert response["batch"]

    def test_after_delete_batched(self):

        from cocktail.persistence import datastore
        from woost.models import DeleteTrigger, Item

        # Declare the trigger
        trigger, response_log = self.make_trigger(
            DeleteTrigger,
            execution_point = "after",
            batch_execution = True
        )

        # Create and insert two items
        item1 = Item()
        item1.insert()
        item2 = Item()
        item2.insert()
        datastore.commit()

        # Delete the items, but abort the transaction. This shouldn't trigger
        # the response.
        item1.delete()
        item2.delete()
        datastore.abort()
        assert not response_log

        # Delete the inserted items and commit the transaction. This should
        # trigger the response just once.
        item1.delete()
        item2.delete()
        datastore.commit()

        assert len(response_log) == 1
        response = response_log[0]
        assert response["trigger"] is trigger
        assert response["items"] == set([item1, item2])
        assert response["user"] is self.user
        assert response["batch"]

    def test_modify_batched_order(self):

        from cocktail.persistence import datastore
        from woost.models import ModifyTrigger, Item, User

        trigger, response_log = self.make_trigger(
            ModifyTrigger,
            execution_point = "after",
            batch_execution = True,
            matching_members = ["woost.models.item.Item.owner"]
        )

        # Modifying a member that is not covered by the trigger should alter
        # the context passed to responses, even if it is modified before the
        # member that actions the response
        item = Item()
        item.insert()
        item.qname = "foo"
        item.owner = User()
        datastore.commit()

        response = response_log[0]
        assert response["context"]["modified_members"] == {
            item: set([(Item.qname, None), (Item.owner, None)])
        }




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			Aug 2009
"""
from cocktail.events import when
from woost.models.trigger import (
    ContentTrigger,
    trigger_responses
)
from woost.extensions.ecommerce.ecommerceorder import ECommerceOrder


class ECommerceOrderCompletedTrigger(ContentTrigger):

    instantiable = True


@when(ECommerceOrder.completed)
def _trigger_ecommerceorder_completed_responses(event):
    trigger_responses(
        ECommerceOrderCompletedTrigger,
        target = event.source
    )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
import cherrypy
from simplejson import dumps
from cocktail.pkgutils import resolve
from cocktail.modeling import cached_getter
from cocktail.translations import (
    translations,
    get_language,
    set_language
)
from cocktail.html import Element
from cocktail import schema
from woost.models import (
    get_current_user,
    ReadPermission
)
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController

class RenderPreviewController(BaseBackOfficeController):

    def __init__(self, *args, **kwargs):
        BaseBackOfficeController.__init__(self, *args, **kwargs)

    def __call__(self, *args, **kwargs):

        preview_language = self.params.read(
            schema.String("preview_language", default = get_language())
        )
        if preview_language:
            set_language(preview_language)

        node = self.stack_node

        get_current_user().require_permission(
            ReadPermission,
            target = node.item
        )

        node.import_form_data(
            node.form_data,
            node.item
        )

        errors = list(node.item.__class__.get_errors(node.item))

        if errors:
            message = Element("div",
                class_name = "preview-error-box",
                children = [
                    translations(
                        "woost.backoffice invalid item preview",
                        preview_language
                    ),
                    Element("ul", children = [
                        Element("li", children = [translations(error)])
                        for error in errors
                    ])
                ]
            )
            message.add_resource("/resources/styles/backoffice.css")
            return message.render_page()
        else:

            self.context.update(
                original_publishable = self.context["publishable"],
                publishable = node.item
            )

            controller = node.item.resolve_controller()

            if controller is None:
                raise cherrypy.NotFound()

            if isinstance(controller, type):
                controller = controller()

            return controller()



#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from cocktail.translations import translations, DATE_STYLE_TEXT
from cocktail.translations.helpers import plural2
from woost.models import Site

# ECommerceExtension
#------------------------------------------------------------------------------
translations.define("ECommerceExtension.pricing",
    ca = u"Preus",
    es = u"Precios",
    en = u"Pricing",
    fr = u"Prix"
)

translations.define("ECommerceExtension.shipping_costs",
    ca = u"Costos d'enviament",
    es = u"Costes de envío",
    en = u"Shipping costs",
    fr = u"Frais d'expédition"
)

translations.define("ECommerceExtension.taxes",
    ca = u"Taxes",
    es = u"Tasas",
    en = u"Taxes",
    fr = u"Taux"
)

translations.define("ECommerceExtension.order_steps",
    ca = u"Passos de la compra",
    es = u"Pasos de la compra",
    en = u"Order steps",
    fr = u"Étapes à suivre pour l'achat"
)

translations.define("ECommerceExtension.payment_types",
    ca = u"Tipus de pagament",
    es = u"Tipos de pago",
    en = u"Payment types",
    fr = u"Type de paiement"
)

translations.define("ECommerceExtension.payment_types-payment_gateway",
    ca = u"Pagament en línia",
    es = u"Pago en línea",
    en = u"On-line payment",
    fr = u"Paiement on-line"
)

translations.define("ECommerceExtension.payment_types-transfer",
    ca = u"Transferència",
    es = u"Transferencia",
    en = u"Transfer",
    fr = u"Transfert"
)

translations.define("ECommerceExtension.payment_types-cash_on_delivery",
    ca = u"Contra reemborsament",
    es = u"Contra reembolso",
    en = u"Cash on delivery",
    fr = u"Contre remboursement"
)

# ECommerceProduct
#------------------------------------------------------------------------------
translations.define("ECommerceProduct",
    ca = u"Producte",
    es = u"Producto",
    en = u"Product",
    fr = u"Produit"
)

translations.define("ECommerceProduct-plural",
    ca = u"Productes",
    es = u"Productos",
    en = u"Products",
    fr = u"Produits"
)

translations.define("ECommerceProduct.product_data",
    ca = u"Dades del producte",
    es = u"Datos del producto",
    en = u"Product data",
    fr = u"Détails sur le produit"
)

translations.define("ECommerceProduct.description",
    ca = u"Descripció",
    es = u"Descripción",
    en = u"Description",
    fr = u"Description"
)

translations.define("ECommerceProduct.price",
    ca = u"Preu",
    es = u"Precio",
    en = u"Price",
    fr = u"Prix"
)

translations.define("ECommerceProduct.weight",
    ca = u"Pes",
    es = u"Peso",
    en = u"Weight",
    fr = u"Poids"
)

translations.define("ECommerceProduct.attachments",
    ca = u"Adjunts",
    es = u"Adjuntos",
    en = u"Attachments",
    fr = u"Pièces jointes"
)

translations.define("ECommerceProduct.purchase_model",
    ca = u"Model de compra",
    es = u"Modelo de compra",
    en = u"Purchase model",
    fr = u"Achat modèle"
)

translations.define("ECommerceProduct.purchases",
    ca = u"Compres",
    es = u"Compras",
    en = u"Purchases",
    fr = u"Achats"
)

translations.define("ECommerceProduct.template",
    ca = u"Plantilla",
    es = u"Plantilla",
    en = u"Template",
    fr = u"Modèle"
)

# ECommerceOrder
#------------------------------------------------------------------------------
translations.define("ECommerceOrder",
    ca = u"Comanda",
    es = u"Pedido",
    en = u"Shop order",
    fr = u"Commande"
)

translations.define("ECommerceOrder-plural",
    ca = u"Comandes",
    es = u"Pedidos",
    en = u"Shop orders",
    fr = u"Commandes"
)

translations.define("ECommerceOrder.customer",
    ca = u"Client",
    es = u"Cliente",
    en = u"Customer",
    fr = u"Client"
)

translations.define("ECommerceOrder.shipping_info",
    ca = u"Direcció d'enviament",
    es = u"Dirección de envío",
    en = u"Shipping address",
    fr = u"Adresse de livraison"
)

translations.define("ECommerceOrder.address",
    ca = u"Adreça",
    es = u"Dirección",
    en = u"Address",
    fr = u"Adresse"
)

translations.define("ECommerceOrder.town",
    ca = u"Població",
    es = u"Población",
    en = u"Town",
    fr = u"Ville"
)

translations.define("ECommerceOrder.region",
    ca = u"Estat o província",
    es = u"Estado o provincia",
    en = u"State or province",
    fr = u"État ou province"
)

translations.define("ECommerceOrder.country",
    ca = u"País",
    es = u"País",
    en = u"Country",
    fr = u"Pays"
)

translations.define("ECommerceOrder.postal_code",
    ca = u"Codi postal",
    es = u"Código postal",
    en = u"Postal code",
    fr = u"Code postal"
)

translations.define("ECommerceOrder.language",
    ca = u"Idioma",
    es = u"Idioma",
    en = u"Language",
    fr = u"Langue"
)

translations.define("ECommerceOrder.status",
    ca = u"Estat",
    es = u"Estado",
    en = u"State",
    fr = u"État"
)

translations.define("ECommerceOrder.status-shopping",
    ca = u"En curs",
    es = u"En curso",
    en = u"Shopping",
    fr = u"En cours"
)

translations.define("ECommerceOrder.status-payment_pending",
    ca = u"Pendent de pagament",
    es = u"Pago pendiente",
    en = u"Payment pending",
    fr = u"Paiement en attente"
)

translations.define("ECommerceOrder.status-accepted",
    ca = u"Acceptada",
    es = u"Aceptada",
    en = u"Accepted",
    fr = u"Accepté"
)

translations.define("ECommerceOrder.status-refund",
    ca = u"Devolució",
    es = u"Devolución",
    en = u"Refund",
    fr = u"Remboursement"
)

translations.define("ECommerceOrder.status-failed",
    ca = u"Cancel·lada",
    es = u"Cancelada",
    en = u"Cancelled",
    fr = u"Annulé"
)

translations.define("ECommerceOrder.purchases",
    ca = u"Contingut de la comanda",
    es = u"Contenido del pedido",
    en = u"Purchases",
    fr = u"Contenu de la commande"
)

translations.define("ECommerceOrder.billing",
    ca = u"Facturació",
    es = u"Facturación",
    en = u"Billing",
    fr = u"Facturation"
)

translations.define("ECommerceOrder.total_price",
    ca = u"Preu",
    es = u"Precio",
    en = u"Price",
    fr = u"Prix"
)

translations.define("ECommerceOrder.pricing",
    ca = u"Modificacions al preu",
    es = u"Modificaciones al precio",
    en = u"Pricing",
    fr = u"Modifications au prix"
)

translations.define("ECommerceOrder.total_shipping_costs",
    ca = u"Costos d'enviament",
    es = u"Costes de envío",
    en = u"Shipping costs",
    fr = u"Frais d'expédition"
)

translations.define("ECommerceOrder.shipping_costs",
    ca = u"Costos d'enviament aplicats",
    es = u"Costes de envío aplicados",
    en = u"Applied shipping costs",
    fr = u"Frais d'expédition appliqués"
)

translations.define("ECommerceOrder.total_taxes",
    ca = u"Taxes",
    es = u"Tasas",
    en = u"Taxes",
    fr = u"Impôts"
)

translations.define("ECommerceOrder.taxes",
    ca = u"Taxes aplicades",
    es = u"Tasas aplicadas",
    en = u"Applied taxes",
    fr = u"Taxes appliquées"
)

translations.define("ECommerceOrder.total",
    ca = u"Total",
    es = u"Total",
    en = u"Total",
    fr = u"Total"
)

translations.define("ECommerceOrder.payment_type",
    ca = u"Tipus de pagament",
    es = u"Tipo de pago",
    en = u"Payment type",
    fr = u"Type de paiement"
)

translations.define("ECommerceOrder.payment_type-payment_gateway",
    ca = u"Pagament en línia",
    es = u"Pago en línea",
    en = u"On-line payment",
    fr = u"Paiment on-line "
)

translations.define("ECommerceOrder.payment_type-transfer",
    ca = u"Transferència",
    es = u"Transferencia",
    en = u"Transfer",
    fr = u"Transfert"
)

translations.define("ECommerceOrder.payment_type-cash_on_delivery",
    ca = u"Contra reemborsament",
    es = u"Contra reembolso",
    en = u"Cash on delivery",
    fr = u"Contre remboursement"
)

translations.define("woost.extensions.ECommerceOrder description for gateway",
    ca = u"Comanda de %s",
    es = u"Pedido de %s",
    en = u"Order at %s",
    fr = u"Commande de %s"
)

# ECommercePurchase
#------------------------------------------------------------------------------
translations.define("ECommercePurchase",
    ca = u"Línia de comanda",
    es = u"Linea de pedido",
    en = u"Purchase",
    fr = u"Achat en ligne"
)

translations.define("ECommercePurchase-plural",
    ca = u"Línies de comanda",
    es = u"Lineas de pedido",
    en = u"Purchases",
    fr = u"Achats en ligne"
)

translations.define("ECommercePurchase.order",
    ca = u"Comanda",
    es = u"Pedido",
    en = u"Order",
    fr = u"Commande"
)

translations.define("ECommercePurchase.product",
    ca = u"Producte",
    es = u"Producto",
    en = u"Product",
    fr = u"Produit"
)

translations.define("ECommercePurchase.quantity",
    ca = u"Quantitat",
    es = u"Cantidad",
    en = u"Quantity",
    fr = u"Quantité"
)

translations.define("ECommercePurchase.billing",
    ca = u"Facturació",
    es = u"Facturación",
    en = u"Billing",
    fr = u"Facturation"
)

translations.define("ECommercePurchase.total_price",
    ca = u"Preu",
    es = u"Precio",
    en = u"Price",
    fr = u"Prix"
)

translations.define("ECommercePurchase.pricing",
    ca = u"Modificacions al preu",
    es = u"Modificaciones al precio",
    en = u"Pricing",
    fr = u"Modifications au prix"
)

translations.define("ECommercePurchase.total_shipping_costs",
    ca = u"Costos d'enviament",
    es = u"Costes de envío",
    en = u"Shipping costs",
    fr = u"Frais d'expédition"
)

translations.define("ECommercePurchase.shipping_costs",
    ca = u"Costos d'enviament aplicats",
    es = u"Costes de envío aplicados",
    en = u"Applied shipping costs",
    fr = u"Frais d'expédition appliqués"
)

translations.define("ECommercePurchase.total_taxes",
    ca = u"Taxes",
    es = u"Tasas",
    en = u"Taxes",
    fr = u"Impôts"
)

translations.define("ECommercePurchase.taxes",
    ca = u"Taxes aplicades",
    es = u"Tasas aplicadas",
    en = u"Applied taxes",
    fr = u"Impôts appliqués"
)

translations.define("ECommercePurchase.total",
    ca = u"Total",
    es = u"Total",
    en = u"Total",
    fr = u"Total"
)

# ECommerceBillingConcept
#------------------------------------------------------------------------------
translations.define("ECommerceBillingConcept",
    ca = u"Concepte de facturació",
    es = u"Concepto de facturación",
    en = u"Billing concept",
    fr = u"Concept de facturation"
)

translations.define("ECommerceBillingConcept-plural",
    ca = u"Conceptes de facturació",
    es = u"Conceptos de facturación",
    en = u"Billing concepts",
    fr = u"Concepts de facturation"
)


translations.define("ECommerceBillingConcept.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title",
    fr = u"Titre"
)

translations.define("ECommerceBillingConcept.enabled",
    ca = u"Activa",
    es = u"Activa",
    en = u"Enabled",
    fr = u"Active"
)

translations.define("ECommerceBillingConcept.start_date",
    ca = u"Data d'inici",
    es = u"Fecha de inicio",
    en = u"Start date",
    fr = u"Date de début"
)

translations.define("ECommerceBillingConcept.end_date",
    ca = u"Data de fi",
    es = u"Fecha de fin",
    en = u"End date",
    fr = u"Date de fin"
)

translations.define("ECommerceBillingConcept.hidden",
    ca = u"Ocult",
    es = u"Oculto",
    en = u"Hidden",
    fr = u"Invisible"
)

translations.define("ECommerceBillingConcept.scope",
    ca = u"Àmbit",
    es = u"Ámbito",
    en = u"Scope",
    fr = u"Champ d'application"
)

translations.define("ECommerceBillingConcept.scope-order",
    ca = u"Comanda",
    es = u"Pedido",
    en = u"Order",
    fr = u"Commande"
)

translations.define("ECommerceBillingConcept.scope-purchase",
    ca = u"Línia de comanda",
    es = u"Linea de pedido",
    en = u"Purchase",
    fr = u"Commande en ligne"
)

translations.define("ECommerceBillingConcept.condition",
    ca = u"Condició",
    es = u"Condición",
    en = u"Condition",
    fr = u"Condition"
)

translations.define("ECommerceBillingConcept.eligible_countries",
    ca = u"Països",
    es = u"Paises",
    en = u"Countries",
    fr = u"Pays"
)

translations.define("ECommerceBillingConcept.eligible_products",
    ca = u"Productes",
    es = u"Productos",
    en = u"Products",
    fr = u"Produits"
)

translations.define("ECommerceBillingConcept.eligible_roles",
    ca = u"Rols",
    es = u"Roles",
    en = u"Roles",
    fr = u"Roles"
)

translations.define("ECommerceBillingConcept.implementation",
    ca = u"Valor",
    es = u"Valor",
    en = u"Value",
    fr = u"Valeur"
)

# Initialization
#------------------------------------------------------------------------------
translations.define("woost.extensions.ecommerce.catalog_page.title",
    ca = u"Botiga",
    es = u"Tienda",
    en = u"Shop",
    fr = u"Boutique"
)

translations.define("woost.extensions.ecommerce.catalog_controller.title",
    ca = u"Controlador de catàleg de productes",
    es = u"Controlador de catálogo de productos",
    en = u"Product catalog controller",
    fr = u"Controleur de catalogue de produits"
)

translations.define("woost.extensions.ecommerce.catalog_template.title",
    ca = u"Plantilla de catàleg de productes",
    es = u"Plantilla de catálogo de productos",
    en = u"Product catalog template",
    fr = u"Modèle de catalogue de produits"
)

translations.define("woost.extensions.ecommerce.basket_page.title",
    ca = u"Cistella de la compra",
    es = u"Cesta de la compra",
    en = u"Shopping basket",
    fr = u"Panier"
)

translations.define("woost.extensions.ecommerce.basket_controller.title",
    ca = u"Controlador per la cistella de la compra",
    es = u"Controlador para la Cesta de la compra",
    en = u"Shopping basket controller",
    fr = u"Controleur de panier"
)

translations.define("woost.extensions.ecommerce.basket_template.title",
    ca = u"Plantilla per cistella de la compra",
    es = u"Plantilla para cesta de la compra",
    en = u"Shopping basket template",
    fr = u"Modèle de panier"
)

translations.define("woost.extensions.ecommerce.checkout_page.title",
    ca = u"Dades de la comanda",
    es = u"Datos del pedido",
    en = u"Checkout",
    fr = u"Détails de la commande"
)

translations.define("woost.extensions.ecommerce.checkout_controller.title",
    ca = u"Controlador de recollida de dades de comanda de la botiga",
    es = u"Controlador de recogida de datos de pedido de la tienda",
    en = u"Shop checkout controller",
    fr = u"Controleur de prise de détails de la commande"
)

translations.define("woost.extensions.ecommerce.checkout_template.title",
    ca = u"Plantilla de recollida de dades de comanda de la botiga",
    es = u"Plantilla de recojida de datos de pedido de la tienda",
    en = u"Shop checkout template",
    fr = u"Modèle de prise de détails de la commande"
)

translations.define("woost.extensions.ecommerce.summary_page.title",
    ca = u"Sumari de la comanda",
    es = u"Sumario del pedido",
    en = u"Order summary",
    fr = u"Resumé de la commande"
)

translations.define("woost.extensions.ecommerce.summary_controller.title",
    ca = u"Controlador de sumari de comanda de la botiga",
    es = u"Controlador de sumario de pedido de la tienda",
    en = u"Shop order summary controller",
    fr = u"Controleur de resumé de la commande"
)

translations.define("woost.extensions.ecommerce.summary_template.title",
    ca = u"Plantilla de sumari de comanda de la botiga",
    es = u"Plantilla de sumario de pedido de la tienda",
    en = u"Shop order summary template",
    fr = u"Modèle de resumé de commande"
)

translations.define("woost.extensions.ecommerce.success_page.title",
    ca = u"Comanda completada",
    es = u"Pedido completado",
    en = u"Order completed",
    fr = u"Commande terminée"
)

translations.define("woost.extensions.ecommerce.success_page.body",
    ca = u"La teva comanda s'ha processat correctament.",
    es = u"Tu pedido se ha procesado correctamente.",
    en = u"Your order has been completed successfully.",
    fr = u"Votre commande a été traitée avec succès."
)

translations.define("woost.extensions.ecommerce.failure_page.title",
    ca = u"Comanda cancel·lada",
    es = u"Pedido cancelado",
    en = u"Order cancelled",
    fr = u"Commande annulée"
)

translations.define("woost.extensions.ecommerce.failure_page.body",
    ca = u"La teva comanda ha estat cancel·lada.",
    es = u"Tu pedido ha sido cancelado.",
    en = u"Your order has been cancelled.",
    fr = u"Votre commande a été annulée."
)

translations.define("woost.extensions.ecommerce.product_controller.title",
    ca = u"Controlador de detall de producte",
    es = u"Controlador de detalle de producto",
    en = u"Shop product detail controller",
    fr = u"Contrôleur de détail du produit"
)

translations.define("woost.extensions.ecommerce.product_template.title",
    ca = u"Plantilla de detall de producte",
    es = u"Plantilla de detalle de producto",
    en = u"Shop product detail template",
    fr = u"Modèle de détail du produit"
)

translations.define("woost.extensions.ecommerce.ecommerceorder_completed_trigger.title",
    ca = u"Enviar resum de la comanda",
    es = u"Enviar resumen del pedido",
    en = u"Send order summary",
    fr = u"Envoyer résumé de la commande"
)

translations.define("woost.extensions.ecommerce.ecommerceorder_completed_emailtemplate.title",
    ca = u"Resum de la comanda",
    es = u"Resumen del pedido",
    en = u"Order summary",
    fr = u"Résumé de la commande"
)

translations.define("woost.extensions.ecommerce.incoming_order.trigger.title",
    ca = u"Enviar resum de la comanda",
    es = u"Enviar resumen del pedido",
    en = u"Send order summary"
)

translations.define("woost.extensions.ecommerce.incoming_order.email_template.title",
    ca = u"Notificació de nova comanda",
    es = u"Notificación de nuevo pedido",
    en = u"New shop order notification"
)

translations.define("woost.extensions.ecommerce.incoming_order.edit_link",
    ca = u"Obrir la comanda",
    es = u"Abrir el pedido",
    en = u"Edit the order"
)

# Notices
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.ecommerce.product_added_notice",
    ca = lambda product:
        u"S'ha afegit <strong>%s</strong> a la cistella"
        % translations(product),
    es = lambda product:
        u"Se ha añadido <strong>%s</strong> a la cesta"
        % translations(product),
    en = lambda product:
        u"<strong>%s</strong> added to the shopping basket"
        % translations(product),
    fr = lambda product:
        u"<strong>%s</strong> ajouté au panier"
        % translations(product)
)


translations.define(
    "woost.extensions.ecommerce.set_quantities_notice",
    ca = u"S'han actualitzat les quantitats",
    es = u"Se han actualizado las cantidades",
    en = u"Product quantities have been updated",
    fr = u"Les montants ont été mis à jour"
)

translations.define(
    "woost.extensions.ecommerce.delete_purchase_notice",
    ca = lambda product:
        u"S'ha tret <strong>%s</strong> de la cistella"
        % translations(product),
    es = lambda product:
        u"Se ha quitado <strong>%s</strong> de la cesta"
        % translations(product),
    en = lambda product:
        u"<strong>%s</strong> has been removed from the shopping basket"
        % translations(product),
    fr = lambda product:
        u"<strong>%s</strong> a été effacé du panier"
        % translations(product)
)

translations.define(
    "woost.extensions.ecommerce.empty_basket_notice",
    ca = u"S'ha buidat la cistella",
    es = u"Se ha vaciado la cesta",
    en = u"The shopping basket has been cleared",
    fr = u"Le panier a été effacé"
)

# ProductView
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.ecommerce.ProductPage.back_link",
    ca = u"Tornar al catàleg",
    es = u"Volver al catálogo",
    en = u"Return to the catalog",
    fr = u"Retour au catalogue"
)

translations.define(
    "woost.extensions.ecommerce.ProductPage.checkout_button",
    ca = u"Continuar la compra",
    es = u"Continuar la compra",
    en = u"Checkout",
    fr = u"Continuer les achats"
)

# OrderStep
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.ecommerce.OrderStep.catalog_button",
    ca = u"Veure més productes",
    es = u"Ver más productos",
    en = u"See more products",
	fr = u"Voire plus products"
)

translations.define(
    "woost.extensions.ecommerce.OrderStep.back_button",
    ca = u"Tornar endarrere",
    es = u"Volver atrás",
    en = u"Go back",
	fr = u"Retourner"
)

translations.define(
    "woost.extensions.ecommerce.OrderStep.proceed_button",
    ca = u"Continuar la compra",
    es = u"Continuar la compra",
    en = u"Proceed",
	fr = u"Procédez"
)

translations.define(
    "woost.extensions.ecommerce.OrderStep.submit_button",
    ca = u"Confirmar la compra",
    es = u"Confirmar la compra",
    en = u"Submit order",
	fr = u"Confirmer l'achat"
)

# BasketPage
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.ecommerce.BasketPage.back_link",
    ca = u"Tornar al catàleg",
    es = u"Volver al catálogo",
    en = u"Return to the catalog",
	fr = u"Retourner au catalogue"
)

# BasketView
#------------------------------------------------------------------------------
translations.define(
    "Basket.product",
    ca = u"Producte",
    es = u"Producto",
    en = u"Product",
	fr = u"Produit"
)

translations.define(
    "Basket.quantity",
    ca = u"Quantitat",
    es = u"Cantidad",
    en = u"Quantity",
	fr = u"Quantité"
)

translations.define(
    "Basket.price",
    ca = u"Preu",
    es = u"Precio",
    en = u"Price",
	fr = u"Prix"
)

translations.define(
    "Basket.subtotal",
    ca = u"Subtotal",
    es = u"Subtotal",
    en = u"Subtotal",
	fr = u"Subtotal"
)

translations.define(
    "Basket.taxes",
    ca = u"Taxes",
    es = u"Tasas",
    en = u"Taxes",
	fr = u"Impôts"
)

translations.define(
    "Basket.total",
    ca = u"Total",
    es = u"Total",
    en = u"Total",
	fr = u"Total"
)

translations.define(
    "SetQuantitiesForm-MinValueError",
    ca = u"No es pot demanar menys d'una unitat d'un producte",
    es = u"No se puede solicitar menos de una unidad de un producto",
    en = u"Can't request less than one unit of a product",
	fr = u"Vous ne pouvez pas demander moins d'une unité d'un produit"
)

translations.define(
    "woost.extensions.ecommerce.BasketView.basket_table_total_header",
    ca = u"Total",
    es = u"Total",
    en = u"Total",
	fr = u"Total"
)

translations.define(
    "woost.extensions.ecommerce.BasketView.delete_purchase_button",
    ca = u"Treure",
    es = u"Quitar",
    en = u"Remove",
	fr = u"Supprimer"
)

translations.define(
    "woost.extensions.ecommerce.BasketView.empty_basket_button",
    ca = u"Buidar la cistella",
    es = u"Vaciar la cesta",
    en = u"Empty the basket",
	fr = u"Vider le panier"
)

translations.define(
    "woost.extensions.ecommerce.BasketView.set_quantities_button",
    ca = u"Establir quantitats",
    es = u"Establecer cantidades",
    en = u"Set quantities",
	fr = u"Fixer les montants"
)

translations.define(
    "woost.extensions.ecommerce.BasketView.empty_basket_notice",
    ca = u"La cistella de la compra està buida.",
    es = u"La cesta de la compra está vacía.",
    en = u"The shopping basket is empty.",
	fr = u"Le panier est vide."
)

translations.define("woost.extensions.ecommerce.BasketView.shipping_costs",
    ca = u"Costos d'enviament",
    es = u"Costes de envío",
    en = u"Shipping costs",
	fr = u"Frais d'expédition"
)

translations.define("woost.extensions.ecommerce.BasketView.taxes",
    ca = u"Taxes",
    es = u"Tasas",
    en = u"Taxes",
	fr = u"Taux"
)

# Discount
#------------------------------------------------------------------------------
translations.define("woost.extensions.ecommerce.Discount.end_date",
    ca = lambda end_date:
        u"fins al " + translations(end_date, style = DATE_STYLE_TEXT),
    es = lambda end_date:
        u"hasta el " + translations(end_date, style = DATE_STYLE_TEXT),
    en = lambda end_date:
        u"until " + translations(end_date, style = DATE_STYLE_TEXT),
	fr = lambda end_date:
        u"jusqu'à " + translations(end_date, style = DATE_STYLE_TEXT)
)

# Basket indicator
#------------------------------------------------------------------------------
translations.define("woost.extensions.ecommerce.BasketIndicator",
    ca = lambda count:
        plural2(
            count,
            u"<strong>1</strong> producte",
            u"<strong>%d</strong> productes" % count
        ) + u" a la cistella",
    es = lambda count:
        plural2(
            count,
            u"<strong>1</strong> producto",
            u"<strong>%d</strong> productos" % count
        ) + u" en la cesta",
    en = lambda count:
        plural2(
            count,
            u"<strong>1</strong> product",
            u"<strong>%d</strong> products" % count
        ) + u" in the shopping basket",
	fr = lambda count:
        plural2(
            count,
            u"<strong>1</strong> produit",
            u"<strong>%d</strong> produits" % count
        ) + u" au panier"
)

# SummaryOrderStep
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.ecommerce.SummaryOrderStep.modify_basket_link",
    ca = u"Modificar el contingut de la comanda",
    es = u"Modificar el contenido del pedido",
    en = u"Edit the shopping basket",
	fr = u"Modifier le contenu de l'ordre"
)

translations.define(
    "woost.extensions.ecommerce.SummaryOrderStep.modify_checkout_link",
    ca = u"Modificar les dades de la comanda",
    es = u"Modificar los datos del pedido",
    en = u"Edit checkout data",
	fr = u"Modifier les données checkout"
)

# AddProductForm
#------------------------------------------------------------------------------
translations.define("woost.extensions.ecommerce.AddProductForm.title",
    ca = u"Afegir a la cistella",
    es = u"Añadir a la cesta",
    en = u"Add to basket",
	fr = u"Ajouter au panier"
)

translations.define("woost.extensions.ecommerce.AddProductForm.submit_button",
    ca = u"Afegir",
    es = u"Añadir",
    en = u"Add",
	fr = u"Ajouter"
)

translations.define(
    "woost.extensions.ecommerce.AddProductForm.product_added_back_button",
    ca = u"Veure més productes",
    es = u"Ver más productos",
    en = u"Keep shopping",
	fr = u"Voir plus de produits"
)

translations.define(
    "woost.extensions.ecommerce.AddProductForm.product_added_forward_button",
    ca = u"Comprar",
    es = u"Comprar",
    en = u"Checkout",
	fr = u"Acheter"
)

# ECommerceOrderCompletedTrigger
#------------------------------------------------------------------------------
translations.define("ECommerceOrderCompletedTrigger",
    ca = u"Disparador de comanda completada",
    es = u"Disparador de pedido completado",
    en = u"Order completed trigger",
	fr = u"Order completed trigger"
)

translations.define("ECommerceOrderCompletedTrigger-plural",
    ca = u"Disparadors de comanda completada",
    es = u"Disparadores de pedido completado",
    en = u"Order completed triggers",
	fr = u"Order completed trigger"
)

# IncomingOrderTrigger
#------------------------------------------------------------------------------
translations.define("IncomingOrderTrigger",
    ca = u"Disparador de nova comanda",
    es = u"Disparador de nuevo pedido",
    en = u"Incoming order trigger"
)

translations.define("IncomingOrderTrigger-plural",
    ca = u"Disparadors de nova comanda",
    es = u"Disparadores de nuevo pedido",
    en = u"Incoming order triggers"
)

# OrderConfirmationMessage
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.OrderConfirmationMessage.product_subtotal_cell",
    ca = u"Subtotal",
    es = u"Subtotal",
    en = u"Subtotal",
	fr = u"Subtotal"
)

translations.define(
    "woost.extensions.OrderConfirmationMessage.product_footer_cell",
    ca = u"Total",
    es = u"Total",
    en = u"Total",
	fr = u"Total"
)

translations.define(
    "woost.extensions.ecommerce.OrderConfirmationMessage.introduction",
    ca = u"Gràcies per la teva compra! Hem rebut la teva comanda "
        u"correctament, i te la farem arribar amb la major brevetat possible. "
        u"A continuació pots veure els productes que has comprat i les dades "
        u"personals i d'enviament que has facilitat. Et recomanem que "
        u"conservis o imprimeixis aquest correu, com a comprovant de la "
        u"compra i per facilitar-ne qualsevol possible gestió.",
    es = u"Gracias por tu compra! Hemos recibido tu pedido correctamente, y "
        u"te lo enviaremos en breve. A continuación te recordamos los "
        u"productos que has comprado y los datos personales y de envío que "
        u"nos has facilitado. Te recomendamos que conserves este correo, como "
        u"comprovante de la compra y para facilitar cualquier possible "
        u"gestión.",
    en = u"Thank you for your purchase! We have received your order, and will "
        u"ship it to you as soon as possible. This message contains a summary "
        u"of the items you purchased, and of your personal and shipping "
        u"information. We recommend you keep or print this message, to use as "
        u"a purchase receipt.",
	fr = u"Merci pour votre achat! Nous avons reçu votre commande "
        u"correctement, et nous alons l'envoyer le plus tôt possible. "
        u"Ci-dessous vous pouvez voir les produits que vous avez acheté et "
        u"les données personnelles que vous nous avez fourni. Nous "
        u"recommandons conserver ou imprimer cet email comme preuve de "
        u"l'achat et pour faciliter toute gestion."
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from woost.models.rendering import (
    content_renderers,
    ImageURIRenderer,
    HTMLRenderer
)
from woost.extensions.vimeo.video import VimeoVideo


class VimeoVideoRenderer(ImageURIRenderer):

    def can_render(self, item):
        return isinstance(item, VimeoVideo)

    def get_item_uri(self, item):
        return item.large_thumbnail_uri

content_renderers.register(VimeoVideoRenderer(), before = HTMLRenderer)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from __future__ import with_statement

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

from itertools import chain
from os.path import join
from tempfile import mkdtemp
import cherrypy
import mimetypes
import pyExcelerator
from cocktail.modeling import (
    getter,
    cached_getter,
    ListWrapper,
    SetWrapper,
    OrderedSet
)
from cocktail.events import event_handler
from cocktail.translations import translations
from cocktail.pkgutils import resolve
from cocktail import schema
from cocktail.schema import ValidationContext
from cocktail.schema.expressions import (
    Expression,
    CustomExpression,
    ExclusionExpression,
    NegativeExpression,
    InclusionExpression,
    Self
)
from cocktail.persistence import datastore
from cocktail.html.datadisplay import (
    SINGLE_SELECTION,
    MULTIPLE_SELECTION
)
from cocktail.controllers import (
    view_state,
    get_parameter,
    CookieParameterSource,
    SessionParameterSource
)
from cocktail.controllers.userfilter import GlobalSearchFilter
from woost.models import (
    Site,
    Language,
    Item,
    UserView,
    changeset_context,
    get_current_user,
    PermissionExpression,
    ReadPermission,
    ReadTranslationPermission
)
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from woost.controllers.backoffice.editstack import (
    EditNode,
    RelationNode,
    SelectionNode
)
from woost.controllers.backoffice.itemcontroller import ItemController
from woost.controllers.backoffice.useractions import get_user_action
from woost.controllers.backoffice.usercollection \
    import BackOfficeUserCollection


class ContentController(BaseBackOfficeController):
    """A controller that handles listings of persistent items."""

    section = "content"
    _item_controller_class = ItemController

    @cached_getter
    def new(self):
        self.context["cms_item"] = None
        return self._item_controller_class()

    def resolve(self, path):

        if not path:
            return self
        else:
            component = path.pop(0)
            try:
                item_id = int(component)
            except ValueError:
                return None
            else:
                item = self.root_content_type.get_instance(item_id)

                if item is None:
                    return None

                self.context["cms_item"] = item
                return self._item_controller_class()

    def __call__(self, *args, **kwargs):

        rel = cherrypy.request.params.get("ItemSelector-select")

        # Open the item selector
        if rel:

            # Load persistent collection parameters before redirecting
            self.user_collection

            pos = rel.find("-")
            root_content_type_name = rel[:pos]
            selection_parameter = str(rel[pos + 1:])

            for content_type in chain([Item], Item.derived_schemas()):
                if content_type.full_name == root_content_type_name:

                    edit_stacks_manager = self.context["edit_stacks_manager"]
                    edit_stack = edit_stacks_manager.current_edit_stack

                    if edit_stack is None:
                        edit_stack = edit_stacks_manager.create_edit_stack()
                        edit_stacks_manager.current_edit_stack = edit_stack

                    node = SelectionNode()
                    node.content_type = content_type
                    node.selection_parameter = selection_parameter
                    edit_stack.push(node)
                    raise cherrypy.HTTPRedirect(node.uri(
                        selection = self.params.read(
                            schema.String(selection_parameter)
                        ),
                        client_side_scripting = self.client_side_scripting
                    ))

        return BaseBackOfficeController.__call__(self, **kwargs)

    @cached_getter
    def action(self):
        """The user action selected by the current HTTP request.
        @type: L{UserAction<woost.controllers.backoffice.useractions.UserAction>}
        """
        return self._get_user_action()

    @cached_getter
    def ready(self):
        return self.action is not None

    def submit(self):
        if self.user_collection.selection_mode == SINGLE_SELECTION:
            selection = [self.user_collection.selection]
        else:
            selection = self.user_collection.selection

        self._invoke_user_action(self.action, selection)

    # Content
    #--------------------------------------------------------------------------
    @cached_getter
    def root_content_type(self):
        """The most basic possible content type for listed items.

        This property is used to constrain the set of eligible content types to
        all types that descend from the indicated type (inclusive).

        @type: L{Item<woost.models.Item>} subclass
        """
        root_content_type = self.stack_content_type

        if root_content_type is None:
            root_content_type_param = self.params.read(
                schema.String("root_content_type")
            )
            root_content_type = resolve(root_content_type_param)

        return root_content_type or Item

    @cached_getter
    def stack_content_type(self):
        """The content type of listed items indicated by the active edit stack.
        @type: L{Item<woost.models.Item>} subclass
        """
        node = self.stack_node

        if node:
            if isinstance(node, SelectionNode):
                return node.content_type

            elif isinstance(node, RelationNode):
                member = node.member

                if isinstance(member, schema.Reference):
                    return member.type
                else:
                    return member.items.type

        return None

    @cached_getter
    def available_languages(self):
        """The list of languages that items in the listing can be displayed in.

        Each language is represented using its two letter ISO code.

        @type: sequence of unicode
        """
        user = get_current_user()
        return [
            language
            for language in Language.codes
            if user.has_permission(
                ReadTranslationPermission,
                language = language
            )
        ]

    @cached_getter
    def user_collection(self):

        user_collection = BackOfficeUserCollection(self.root_content_type)

        if self.edit_stack and isinstance(self.stack_node, RelationNode):
            user_collection.default_type = \
                self.stack_node.member.selector_default_type

        user_collection.available_languages = self.available_languages
        user_collection.selection_mode = self.selection_mode
        user_collection.default_order = \
            [NegativeExpression(Item.last_update_time)]

        # Parameter persistence
        prefix = self.persistence_prefix
        duration = self.persistence_duration

        user_collection.set_parameter_source("type",
            SessionParameterSource(key_prefix = prefix)
        )

        type_prefix = user_collection.type.full_name
        if prefix:
            type_prefix += "-" + prefix

        user_collection.persistence_prefix = type_prefix
        user_collection.persistent_source = psource = SessionParameterSource(
            key_prefix = type_prefix
        )

        user_collection.set_parameter_source("content_view", psource)
        user_collection.set_parameter_source("members", psource)
        user_collection.set_parameter_source("order", psource)
        user_collection.set_parameter_source("grouping", psource)
        user_collection.set_parameter_source("filter", psource)
        user_collection.set_parameter_source("page", psource)
        user_collection.set_parameter_source("page_size", psource)
        user_collection.set_parameter_source("expanded", psource)
        user_collection.set_parameter_source("language",
            CookieParameterSource(
                cookie_naming = "visible_languages",
                cookie_duration = duration
            )
        )

        # Exclude instances of invisible types
        def hide_invisible_types(content_type):
            if not content_type.visible:
                exclusion = ExclusionExpression(Self, content_type.keys)
                exclusion.by_key = True
                user_collection.add_base_filter(exclusion)
            else:
                for descendant_type \
                in content_type.derived_schemas(recursive = False):
                    hide_invisible_types(descendant_type)

        hide_invisible_types(user_collection.type)

        # Exclude edit drafts
        user_collection.add_base_filter(Item.draft_source.equal(None))

        node = self.stack_node

        if node and isinstance(node, RelationNode):

            relation = node.member
            is_collection = isinstance(relation, schema.Collection)
            edit_node = self.edit_stack[-2]
            excluded_items = set()

            # Exclude items that are already contained on an edited collection
            if is_collection:
                excluded_items.update(
                    schema.get(edit_node.form_data, relation)
                )

            if excluded_items:
                user_collection.add_base_filter(
                    ExclusionExpression(Self, excluded_items)
                )

            # Add relation constraints
            if relation.enumeration:
                enumeration = relation.resolve_constraint(
                    relation.enumeration,
                    ValidationContext(
                        edit_node.item.__class__,
                        edit_node.item,
                        persistent_object = edit_node.item
                    )
                )
                user_collection.add_base_filter(
                    InclusionExpression(
                        Self, enumeration
                    )
                )

            for constraint in relation.get_constraint_filters(edit_node.item):
                user_collection.add_base_filter(constraint)

        # Filter unauthorized items
        user_collection.add_base_filter(
            PermissionExpression(get_current_user(), ReadPermission)
        )

        return user_collection

    @cached_getter
    def selection_mode(self):
        if self.edit_stack \
        and isinstance(self.stack_node, RelationNode) \
        and isinstance(self.stack_node.member, schema.Reference):
            return SINGLE_SELECTION
        else:
            return MULTIPLE_SELECTION

    @cached_getter
    def search_expanded(self):
        return bool(
            self.user_collection.user_filters
            or self.params.read(schema.Boolean("search_expanded"))
        )

    @cached_getter
    def user_views(self):

        user = get_current_user()
        views = OrderedSet()

        # Role views
        for role in user.iter_roles():
            views.extend(role.user_views)

        # User views
        views.extend(UserView.select(filters = [
            UserView.owner.equal(user)
        ]))

        return views

    # Parameter persistence
    #--------------------------------------------------------------------------
    @cached_getter
    def persistence_prefix(self):
        stack = self.edit_stack
        return stack.to_param() if stack else ""

    @cached_getter
    def persistence_duration(self):
        node = self.stack_node
        return (
            None
            if node and isinstance(node, (RelationNode, SelectionNode))
            else self.settings_duration
        )

    # Rendering
    #--------------------------------------------------------------------------
    @cached_getter
    def view_class(self):
        if self.edit_stack:
            return "woost.views.BackOfficeItemSelectorView"
        else:
            return "woost.views.BackOfficeContentView"

    @cached_getter
    def output(self):
        output = BaseBackOfficeController.output(self)
        output.update(
            user_collection = self.user_collection,
            available_languages = self.available_languages,
            selection_mode = self.selection_mode,
            root_content_type = self.root_content_type,
            search_expanded = self.search_expanded,
            user_views = self.user_views
        )
        return output

    # TODO: Move MS Excel rendering to an extension
    allowed_rendering_formats = (
        BaseBackOfficeController.allowed_rendering_formats
        | frozenset(["msexcel"])
    )

    def render_msexcel(self):

        content_type = mimetypes.types_map.get(".xls")
        cd = 'attachment; filename="%s"' % (
            translations(self.user_collection.type.name + "-plural") + ".xls"
        )
        languages = self.params.read(
            schema.Collection("language", items = schema.String())
        )

        cherrypy.response.headers['Content-Type'] = content_type
        cherrypy.response.headers["Content-Disposition"] = cd

        buffer = StringIO()
        self.user_collection.export_file(
            buffer, mime_type = content_type, languages = languages
        )
        return buffer.getvalue()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cStringIO import StringIO
import cherrypy
import Image
from woost.models import (
    Item,
    get_current_user,
    ModifyPermission
)
from woost.models.rendering import content_renderers
from woost.controllers.basecmscontroller import BaseCMSController


class ImageEffectsController(BaseCMSController):

    def __call__(self, id, *effects_chain, **kwargs):

        # Get the requested item or content type, validate access
        item = None

        try:
            id = int(id)
        except ValueError:
            for cls in Item.schema_tree():
                if cls.full_name == id:
                    item = cls
                    break
        else:
            item = Item.get_instance(int(id))

        if item is None:
            raise cherrypy.NotFound()

        get_current_user().require_permission(
            ModifyPermission,
            target = item
        )

        image = content_renderers.render(
            item,
            effects_chain,
            apply_item_effects = (kwargs.get("override") != "true")
        )

        if isinstance(image, basestring):
            image = Image.open(image)

        buffer = StringIO()
        cherrypy.response.headers["Content-Type"] = "image/png"
        image.save(buffer, "JPEG" if image.mode == "CMYK" else "PNG")
        return buffer.getvalue()

